function [PLayer, TLayer,  FLayer, t, e] = addlayer(P, t, z)
%ADDLAYER - Creates tetrahedral mesh for base layer using Delaunay tesselation
%
% Syntax:
%   [PLayer, TLayer,  FLayer, t, e] = addlayer(P, t, z);
%
% Description:
%   ADDLAYER creates a one-layer tetrahedral mesh.  Given the structure of
%   the base planar mesh and the thickness of the layer to be created,
%   ADDLAYER creates the nodes at the top of the layer and performs a
%   three-dimensional Delaunay tesselation of the layer.  It returns the
%   top layer nodes, the tetrahedra, and the faces of the resulting 3-D
%   mesh.
%
% Input:
%   P - array of nodes for the base planar mesh (from STRUCT2D) 
%   t - array of triangles for the base planar mesh (from STRUCT2D)
%   z - layer thickness
%
% Output:
%   PLayer - array of top layer nodes
%   TLayer - array of tetrahedra
%   FLayer - array of layer faces
%   t      - updated t-array  (after Delaunay)
%   e      - edge array (for vertical via faces) 

%   MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
%   Authors: A. Marut, S. Makarov

eps     =   1e-5*z;

PLayer  =   [P(1,:)*(1 + eps); P(2,:)*(1 + eps); z*ones(1,size(P,2))];        
P       =   [P PLayer];         %   Add top

%   Mesh conditioner
P(1,:)  =   P(1,:) - eps*P(3,:);
P(2,:)  =   P(2,:) - eps*P(3,:) - eps*P(1,:);

%   Apply Delaunay tessellation in 3D
TLayer      = delaunayn(P',{'Qt','Qbb','Qc','Qz'});    
TLayer      = TLayer'; TLayer = sort(TLayer,1);    

%   Remove tetrahedra that are not part of the original mesh
CenterTX = mean(reshape(P(1,TLayer), size(TLayer)));
CenterTY = mean(reshape(P(2,TLayer), size(TLayer)));
should_exist = false(1,size(TLayer,2));
for tri = t    
    % Find indices of points inside the mth triangle
    ind     = inpolygon( CenterTX, CenterTY, P(1, tri), P(2, tri));
    should_exist = should_exist | ind;
end
TLayer(:,~should_exist) = [];

[T, FacesInner, FacesBoundary, Edges, ATB, ATI] = face2(P, TLayer);

FLayer      =   [FacesBoundary FacesInner];

%   Update FLayer
CenterFZ    = mean(reshape(P(3, FLayer), size(FLayer)));
bottom      = find(abs(CenterFZ)< 10*eps);
top         = find(abs(CenterFZ - z)< 10*eps);
middle      = setdiff(1:size(FLayer, 2),[top bottom]);
FLayer      = [FLayer(:, bottom) FLayer(:, top) FLayer(:, middle)];
t           =  FLayer(:, 1:length(bottom));

%   Update e
CenterEZ    = mean(reshape(P(3, Edges), size(Edges)));
e           = Edges(:,find(abs(CenterEZ)< 10*eps));
e(3,:)      = e(2,:);

