function [sel_t] = addvia(P0, e0, tLayer1, numbers);
%ADDVIA - Selects via metal triangles for a group of layers 
%
% Syntax:
%   [sel_t] = addvia(P0, e0, tLayer1, numbers);
%
% Description:
%   ADDVIA calls TRISELECT and adds selected edges.
%
% Input:
%   P0 - array of nodes for the volumetric mesh - base layer 
%   e0 - array of edges (triangles) for the base planar mesh
%   tLayer1 - array of faces for the volumetric mesh - base layer 
%   numbers - array of layer numbers
%
% Output:
%   sel_t - cell array of index arrays of triangles to be removed from the
%           set of faces [index arrays of edges of the via], one layer per
%           cell.  A value of {NaN} (i.e., a cell array containing the
%           value NaN) is currently used to indicate that the user closed
%           the window.
%
% See also: TRISELECT.

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, A. Marut

if length(numbers)==0
    str = ' nothing'; % shouldn't be called with numbers empty anyway
    sel_t = [1:size(tLayer1, 2)];
    return;
elseif length(numbers)==1
    str = [ ' layer #' num2str(numbers)];
else
    str = [ ' layers ' num2str(numbers)];
end

%   Select triangles - via
string          = strcat('Select via metal faces of ',str); 
Sel             = triselect(P0, e0, string);
s               = size(tLayer1, 2);
if isnan(Sel)
    sel_t       = {NaN}; %1:s;
else
    sel_t       = [];
    for m = 1:s
        CFaceXY(1:2,m) =  sum(P0(1:2, tLayer1(1:3,m)),2)/3;
    end    
    for m = 1:length(Sel)
        p1      = P0(1:2, e0(1, Sel(m)));
        p2      = P0(1:2, e0(2, Sel(m)));
        length0 = sqrt(dot(p1-p2, p1-p2));
        vector1 = + CFaceXY - repmat(p1, [1 s]);
        vector2 = - CFaceXY + repmat(p2, [1 s]);
        length1 = sqrt(dot(vector1, vector1));
        length2 = sqrt(dot(vector2, vector2));
        lengthn = (length1 + length2)/length0;
        ind     = find(abs(lengthn-1)<1e-3);
        sel_t   = [sel_t ind];      
    end 
    sel_t       = setdiff([1:size(tLayer1, 2)], sel_t);
    temp = {};
    for n=numbers
        temp{end+1} = sel_t;
    end
    sel_t = temp;
end