function Edges = edgesort(Edges, const, ATE)
%EDGESORT - Sorts the edges by their dielectric constant.
%
% Syntax:
%   Edges = edgesort(Edges, const, ATE);
%
% Description:
%   EDGESORT sorts an array of dielectric edges in order of descending
%   dielectric constant.  EDGESORT assigns to each edge a dielectric
%   constant that is the arithmetic mean of the dielectric constants of all
%   adjacent tetrahedra; the tetrahedra are weighted equally.
%
% Input:
%   Edges - array of edges for the volumetric mesh
%   const - structure array with EM data (from EMDATA)
%   ATE - cell array of the tetrahedra to which each edge belongs
%
% Output:
%   Edges - array of edges for volumetric mesh, in sorted order

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: A. Marut, S. Kulkarni

EdgesTotal = size(Edges,2);
edge_epsilons = zeros(1, EdgesTotal);
for i = 1:EdgesTotal
    index_T = ATE{i};
    surrounding_epsilons = const.Epsilon_r(index_T);
    edge_epsilons(i) = mean(surrounding_epsilons);
end
        
% Sort edge_epsilons and store the destination indices
[edge_epsilons ind] = sort(edge_epsilons, 2, 'descend');
% Reorder the edges according to these indices
Edges = Edges(:,ind);


% % Find the z-coordinate of the midpoint of each edge
% EdgeCenter(:,1:EdgesTotal)= (P(3,Edges(1,1:EdgesTotal))+ P(3,Edges(2,1:EdgesTotal)))/2;
% [Y, ind]= sort(EdgeCenter);
% Edges = Edges(:,ind); % Edges belonging to a particular group are stacked together.
% EdgeCenter = EdgeCenter(ind);
% 
% % Create an array containing the dielectric constant of each edge; the dielectric
% % constant of an edge between two layers is the average of the dielectric
% % constants of those two layers.
% 
% N = layerdata.num_layers;
% layer_z_values = cumsum(layerdata.thicknesses);
% 
% if N > 1
%     % partition contains two entries for every boundary between two layers
%     partition = zeros(1, 2*(N-1));
%     partition(1:2:2*N-3) = layer_z_values(1:end-1) .* (1 - eps * (1:N-1));  % Odd elements are just below the layer boundary
%     partition(2:2:2*N-2) = layer_z_values(1:end-1) .* (1 + eps * (1:N-1));  % Even    "     "    "  above  "    "       "   
% 
%     codebook = zeros(1, 2*N - 1);
%     codebook(1:2:2*N-1) = layerdata.epsilons; % Odd elements are the epsilons within each layer
%     codebook(2:2:2*N-2) = (codebook(1:2:2*N-3) + codebook(3:2:2*N-1)) / 2; % Even elements are the epsilons between layers
% 
%     [indx  edge_epsilons] = quantiz(EdgeCenter, partition, codebook);
%     
% else
%     edge_epsilons = layerdata.epsilons(1) * ones(1,EdgesTotal);
% end
