function const = emdata(P, T, layers, relative_epsilons, tan_deltas)
%EMDATA - Creates necessary EM data
%
% Syntax:
%   const = emdata(P, T, layers, relative_epsilons, tan_deltas);
%
% Description:
%   CONST creates a data structure that contains electromagnetic constants.
%   The structure contains numerical values of physical constants as well
%   as electromagnetic material data for each tetrahedron.
%
% Input:
%   P - array of nodes for volumetric mesh
%   T - array of tetrahedra for volumetric mesh
%   layers - structure array (from LAYERS)
%   relative_epsilons - array of permittivities of each tetrahedron
%   tan_deltas - array of dielectric loss tangents of each tetrahedron
%
% Output:
%   const - structure array containing the following fields:
%       epsilon - the permittivity of free space
%       mu - the permeability of free space
%       c - the speed of light in a vacuum
%       eta - the impedance of free space
%       isdielectric - logical variable stating whether the dielectric permittivity
%           differs from that of free space
%       Epsilon_r - array containing the real dielectric permittivity [F/m] of
%           each tetrahedron
%       tan_delta - array containing the dielectric loss tangent tan(delta) of each
%           tetrahedron (tan(delta) = eps_i/eps_r; eps = eps_r - j*eps_i)

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, S. Kulkarni, A. Marut


%   Dielectric data - vacuum
const.epsilon       = 8.85418782e-012;  %  ANSOFT HFSS value 
const.mu            = 1.25663706e-006;  %  ANSOFT HFSS value
const.c             = 1/sqrt(const.epsilon*const.mu);
const.eta           = sqrt(const.mu/const.epsilon);
const.isdielectric  = sum((layers.epsilons - 1) .* layers.thicknesses) ...
                    / sum(layers.thicknesses) >0; 
                    %   logical - if dielectric contrast is present at all
                
%   Calculate const.Epsilon_r for each tetrahedron
const.Epsilon_r = const.epsilon * relative_epsilons;
const.tan_delta = tan_deltas;

% Z               = P(3,:);           
% ZC              = mean(Z(T));       
% Epsilon_r       = zeros(1,size(T,2));
% tan_delta       = zeros(1,size(T,2));
% epsilons        = layers.epsilons;
% tan_deltas      = layers.tan_deltas;
% [ZC I]          = sort(ZC);         %   put the centroids in order so we can go through the layer boundaries in order
% zi              = 1;                %   index in z_values of the z coordinate of the layer boundary
% z_values        = cumsum(layers.thicknesses);
% z_upper         = z_values(zi);     %   upper bound on values of centroids in layer zi
% curr_epsilon    = epsilons(zi);     %   current value of epsilon_r for this layer
% curr_tan_delta  = tan_deltas(zi);   %   current value of tan_delta for this layer
% for i = 1:size(T,2)
%     if ZC(i) > z_upper              %   if all centroids in this layer have been seen and the next layer has been reached
%         zi              = zi + 1;
%         z_upper         = z_values(zi);
%         curr_epsilon    = epsilons(zi);
%         curr_tan_delta  = tan_deltas(zi);
%     end
%     Epsilon_r(I(i))     = curr_epsilon;
%     tan_delta(I(i))     = curr_tan_delta;
% end
% const.Epsilon_r = const.epsilon*Epsilon_r;
% const.tan_delta = tan_delta;
