function [T, Faces, FacesBoundary, FacesNontrivial, Edges, AT, ATE, const]  = face1(P, T, const);
%FACE1 - Outputs faces/edges and faces with nontrivial dielectric contrast for tetrahedral mesh
%
% Syntax:
%   [T, Faces, Edges, FacesBoundary, FacesNontrivial, AT, ATE, const]  = face1(P, T, const);
%
% Description:
%   FACE1 calculates faces/edges and faces with nontrivial dielectric contrast for
%   tetrahedral mesh
%
% Input:
%   P - array of nodes for volumetric mesh 
%   T - array of tetrahedra for volumetric mesh 
%   const - structure array with EM data (from EMDATA)
%
% Output:
%   T - array of tetrahedra for the volumetric mesh (in canonical form)
%   Faces - array of faces for the volumetric mesh
%   FacesBoundary - array of boundary dielectric faces
%   FacesNontrivial - array of nontrivial dielectric faces
%       (either boundary faces or faces with nonzero dielectric contrast)
%   Edges - array of dielectric edges
%   AT - array of adjacent tetrahedra for every dielectric face
%   ATE - cell array of tetrahedra to which each edge belongs
%   const - structure array with EM data (from EMDATA)


% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, S. Kulkarni



[T, FacesInner, FacesBoundary, Edges, ATB, ATI, ATE, const] = face2(P, T, const);
Faces           =   [FacesBoundary FacesInner];
AT              =   [ATB ATI];
FacesNontrivial = length(FacesBoundary);

%   Redefine nontrivial faces 
FacesNontrivial = [];
for m = 1:size(Faces,2)
    if AT(1, m) == 0
        FacesNontrivial = [FacesNontrivial m];
    else
        ContrastDiff1 = abs(const.Epsilon_r(AT(1, m))-const.Epsilon_r(AT(2, m)));
        if (ContrastDiff1)>0
            FacesNontrivial = [FacesNontrivial m];
        end
    end
end
FacesBoundary   = Faces(:, FacesNontrivial);
ATBoundary      = AT(:, FacesNontrivial);
temp            = setdiff([1:size(Faces,2)], FacesNontrivial);
FacesInner      = Faces(:,temp);
ATInner         = AT(:,temp);
Faces           = [FacesBoundary FacesInner];
AT              = [ATBoundary ATInner];
FacesNontrivial = length(FacesNontrivial);

