function varargout = face2(varargin);
%FACE2 - Creates faces/edges for tetrahedral mesh
%
% Syntax:
%   [T, FacesInner, FacesBoundary, Edges, ATB, ATI, ATE] = face2(P, T);
%   [T, FacesInner, FacesBoundary, Edges, ATB, ATI, ATE, const] = face2(P, T, const);
%
% Description:
%   FACE2 calculates faces, edges, and tetrahedra adjacent to faces 
%   for tetrahedral mesh
%
% Input:
%   P - array of nodes for the volumetric mesh
%   T - array of tetrahedra for the volumetric mesh 
%   const - structure array with EM data (from EMDATA)
%
% Output:
%   T - array of tetrahedra for the volumetric mesh (in canonical order)
%   FacesInner - array of inner dielectric faces for the volumetric mesh 
%   FacesBoundary - array of boundary dielectric faces for the
%       volumetric mesh
%   Edges - array of dielectric edges
%   ATB - array of the adjacent tetrahedron for each boundary face
%   ATI - array of the two adjacent tetrahedra for each inner face
%   ATE - cell array of the tetrahedra to which each edge belongs
%   const - structure array with EM data (from EMDATA)
% The outputs are all in canonical form: each column is sorted internally,
% and all columns are sorted as a group.

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Author: A. Marut

P = varargin{1};
T = varargin{2};
if nargin == 3
    const = varargin{3};
end

%   Put T in canonical order
T = sort(T,1);
[T I] = sortrows(T');
T = T';
if exist('const')
    const.Epsilon_r = const.Epsilon_r(I);
    const.tan_delta = const.tan_delta(I);
end

%   Find all the edges in canonical form
[Edges ATElong] = findfaces(T(1:4,:), 'edges');
EdgesUnique = unique(Edges', 'rows')';
% make a list of all tetrahedra to which each edge belongs
ATE = cell(1, length(EdgesUnique));
j = 1;
for i = 1:size(Edges, 2);
    curr_edge = Edges(:,i);
    if i ~= 1 && any(curr_edge ~= last_edge)
        j = j + 1;
    end
    ATE{j} = [ATE{j} ATElong(i)];
    last_edge = curr_edge;
end
Edges = EdgesUnique;


%   Find all the 2-faces and associate each with a tetrahedron
[Faces ATlong] = findfaces(T(1:4,:), 'faces');

%   Make the faces unique, and don't lose the association with the tets
[Faces I J] = unique(Faces', 'rows');
Faces = Faces';
%   J has the indices of where each face went, so use it to make the
%   tetrahedra follow their faces.
AT = zeros(2,size(Faces,2));
for i = 1:size(ATlong,2)
    TetIndex = ATlong(i);
    ATIndex = J(i);
    if AT(1,ATIndex) == 0
        AT(1,ATIndex) = TetIndex;
    elseif AT(2,ATIndex) == 0
        AT(2,ATIndex) = TetIndex;
    else
        error 'FACE2: found a facet attached to more than two tetrahedra!';
    end
end
AT = sort(AT);

% Find all boundary faces
FacesBoundary = Faces(:,any(AT == 0));
ATB = AT(:,any(AT == 0));
% Find all inner faces
FacesInner    = Faces(:,all(AT ~= 0));
ATI = AT(:,all(AT ~= 0));

varargout{1} = T;
varargout{2} = FacesInner;
varargout{3} = FacesBoundary;
varargout{4} = Edges;
varargout{5} = ATB;
varargout{6} = ATI;
varargout{7} = ATE;
if nargout == 8
    varargout{8} = const;
end
