function varargout = layers(varargin)
%LAYERS - Obtains from the user the properties of every layer for the resonator structure
%
% Syntax:
%   layerdata = layers;
%
% Description:
%   LAYERS displays a window that prompts the user to enter the properties
%   of the resonator by layer.  Because the user may want to define several
%   layers with the same physical properties, such layers may be defined as
%   a group.  Therefore, the user first enters the number of layer groups.
%   All layers within a group have the same thickness, dielectric constant,
%   dielectric loss tangent, and cross section, but the layers do not need
%   to be contiguous.  For each layer group, the user enters a list of the
%   layers in the group; each layer is denoted by an integer from 1 to the
%   total number of layers.  In addition, the user enters the thickness,
%   dielectric constant, and dielectric loss tangent of the layers in the
%   group.
%
% Output:
%   layerdata - structure containing the following fields:
%       exitmode - string describing how the user exited the program.  It
%           is either 'ok', 'cancel', 'close', or 'error'.
%       If exitmode is 'ok', then layerdata also contains the following:
%       total_thickness - the total thickness of the resonator
%       num_layers - the number of layers in the resonator
%       num_layer_groups - the number of groups of layers
%       groupings - cell array of arrays of the layer numbers in each group
%       thicknesses - array of the thickness (in meters)        of each layer
%       epsilons -    array of the relative dielectric constant of each layer
%       tan_deltas -  array of the dielectric loss tangent      of each layer

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Author: A. Marut


% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @layers_OpeningFcn, ...
                   'gui_OutputFcn',  @layers_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before layers is made visible.
function layers_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to layers (see VARARGIN)

% Choose default command line output for layers
handles.exitmode = 'error';

% Constants
handles.THICKNESS_TOLERANCE_RATIO = 1e-6;

handles.FIRST_LABEL_IN_FRAME_POSITION       = [ 28  464  185  15];
handles.FIRST_TEXTFIELD_IN_FRAME_POSITION   = [220  464   87  21];
handles.FIRST_COMMENT_IN_FRAME_POSITION     = [ 28  444  279  15];
handles.NEXT_COMPONENT_UPPER_FRAME_POSITION = [  0  -53    0   0];
handles.NEXT_COMPONENT_LOWER_FRAME_POSITION = [  0  -28    0   0];

if exist('layers.mat', 'file')
    load layers;
else
    num_layer_groups = 1;
    num_layer_groups_string = '1';
    groupings   = {'1'};
    thicknesses = {'0'};
    epsilons    = {'1'};
    tan_deltas  = {'0'};
    total_thickness = '0';
end
handles.num_layer_groups = num_layer_groups;
handles.num_layer_groups_string = num_layer_groups_string;
handles.num_layer_groups_stored = num_layer_groups;

set(handles.num_layer_groups_edit, 'String', num_layer_groups_string);

handles.grouping_labels     = [];   handles.grouping_boxes   = [];
handles.thickness_labels    = [];   handles.thickness_boxes  = [];
handles.epsilon_labels      = [];   handles.epsilon_boxes    = [];
handles.tan_delta_labels    = [];   handles.tan_delta_boxes  = [];

for i=1:num_layer_groups
    handles = create_frame_component_group(handles, i);
    set(handles.grouping_boxes(i),  'String', groupings{i});
    set(handles.thickness_boxes(i), 'String', thicknesses{i});
    set(handles.epsilon_boxes(i),   'String', epsilons{i});
    set(handles.tan_delta_boxes(i), 'String', tan_deltas{i});
end

set(handles.total_thickness_edit, 'String', total_thickness);

handles.layer_group_1 = 1; % the layer group described in the upper half of the frame is the first layer group
handles.old_slider_value = -handles.layer_group_1;
handles = update_slider(handles);
handles = update_frame(handles);

comment_strs = ['For example, [1  2  3  7] or [4 5].  Brackets are optional.'; ...
                'Dimensions are millimeters.  For example, 2.5 is 2.5 mm.   '; ...
                'Relative permittivity (dimensionless), e.g. 4.4.           '; ...
                'Dielectric loss tangent (dimensionless), e.g. 0.15.        '];
position = handles.FIRST_COMMENT_IN_FRAME_POSITION;
for i=1:4
    comment_label = create_label(comment_strs(i,:));
    set(comment_label, 'Position', position, 'Visible', 'on');
    position = position + handles.NEXT_COMPONENT_UPPER_FRAME_POSITION;
end

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes layers wait for user response (see UIRESUME)
uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = layers_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
if isempty(handles)
    % user clicked the X
    varargout{1} = struct('exitmode', 'close');
elseif ~strcmp(handles.exitmode, 'ok')
    varargout{1} = struct('exitmode', handles.exitmode);    
    close gcf;
else
    thicknesses = [0];
    epsilons = [0];
    tan_deltas = [0];
    for i=1:handles.num_layer_groups
        thicknesses(handles.groupings{i}) = handles.thicknesses(i);
        epsilons   (handles.groupings{i}) = handles.epsilons(i);
        tan_deltas (handles.groupings{i}) = handles.tan_deltas(i);
    end
    varargout{1} = struct( ...
        'exitmode',         handles.exitmode,                           ...
        'total_thickness',  handles.total_thickness,                    ...
        'num_layers',       handles.num_layers,                         ...
        'num_layer_groups', handles.num_layer_groups,                   ...
        'groupings',        {handles.groupings},                        ...
        'thicknesses',      thicknesses,                                ...
        'epsilons',         epsilons,                                   ...
        'tan_deltas',       tan_deltas      );
    close gcf;
end


function num_layer_groups_edit_Callback(hObject, eventdata, handles)
% hObject    handle to num_layer_groups_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% The program requires the user to click 'Update' to change the number of
% layers.  Therefore, this function does nothing.


% --- Executes on button press in update_btn.
function update_btn_Callback(hObject, eventdata, handles)
% hObject    handle to update_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.num_layer_groups_string = get(handles.num_layer_groups_edit, 'String');
handles.num_layer_groups = str2num(handles.num_layer_groups_string);

% enlarge the lists of uicontrols if necessary
for i=handles.num_layer_groups_stored+1:handles.num_layer_groups
    handles = create_frame_component_group(handles, i);
    set(handles.grouping_boxes(i),  'String', '');
    set(handles.thickness_boxes(i), 'String', '0');
    set(handles.epsilon_boxes(i),   'String', '1');
    set(handles.tan_delta_boxes(i), 'String', '0');
end

handles.num_layer_groups_stored = max(handles.num_layer_groups_stored, handles.num_layer_groups);

handles.layer_group_1 = min(handles.layer_group_1, max(1, handles.num_layer_groups - 1));
% min() ensures that, if the number of layer groups decreased, the two
% layer groups being displayed are within range.
% max() ensures that, if there is now only one layer group, the first layer
% group (and not the nonexistent zeroth layer group) is at the top of the
% frame.

handles = update_slider(handles);
handles = update_frame(handles);
guidata(hObject,handles);


% --- Executes on button press in calc_total_thickness_btn.
function calc_total_thickness_btn_Callback(hObject, eventdata, handles)
% hObject    handle to calc_total_thickness_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.warning_text, 'String', '');
[handles warning_string] = check_layer_occurrences(handles);
if ~isempty(warning_string)
    set(handles.warning_text, 'String', ['Warning: ' warning_string]);
end

total_thickness = calc_total_thickness(handles);
set(handles.total_thickness_edit, 'String', num2str(total_thickness));

guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function slider1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background, change
%       'usewhitebg' to 0 to use default.  See ISPC and COMPUTER.
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function slider1_Callback(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% If the user drags the slider, its value probably won't be an integer.  This fixes that.
handles.layer_group_1 = round(-get(hObject, 'Value'));
set(hObject, 'Value', -handles.layer_group_1);

handles = update_slider(handles);
handles = update_frame(handles);
handles.old_slider_value = get(handles.slider1, 'Value');

guidata(hObject,handles);


function handles = update_slider(handles)
if handles.num_layer_groups <= 2
    set(handles.slider1, 'Visible', 'off');
else
    slider_min = -(handles.num_layer_groups - 1);       % The lowest position of the slider is when the last two groups are visible.
    slider_step(1) = 1/(handles.num_layer_groups - 2);  % When the user clicks an arrow, move one layer group at a time.
    slider_step(2) = 2*slider_step(1);                  % When the user clicks above or below the bar, move two layer groups at a time.
    set(handles.slider1, 'Min', slider_min, 'Max', -1, 'SliderStep', slider_step);
    set(handles.slider1, 'Value', -handles.layer_group_1, 'Visible', 'on');
end


function handles = update_frame(handles)
% Refreshes the view of the frame

% First set other things invisible:
i = -handles.old_slider_value;
handles = set_component_group_visibility(handles, i, 'off');
if i+1 <= handles.num_layer_groups_stored
    handles = set_component_group_visibility(handles, i+1, 'off');
end

% Position the components in the upper half of the frame:
i = handles.layer_group_1;
handles = place_component_group(handles, i, 'upper');
handles = set_component_group_visibility(handles, i, 'on');

i = i + 1;
if i <= handles.num_layer_groups
    handles = place_component_group(handles, i, 'lower');
    handles = set_component_group_visibility(handles, i, 'on');
end


function handles = place_component_group(handles, i, which_half)
label_position = handles.FIRST_LABEL_IN_FRAME_POSITION;
textfield_position = handles.FIRST_TEXTFIELD_IN_FRAME_POSITION;
shift = handles.NEXT_COMPONENT_UPPER_FRAME_POSITION;

if strcmpi(which_half, 'lower')
    label_position = label_position + 4 * handles.NEXT_COMPONENT_UPPER_FRAME_POSITION - [0 10 0 0];
    textfield_position = textfield_position + 4 * handles.NEXT_COMPONENT_UPPER_FRAME_POSITION - [0 10 0 0];
    shift = handles.NEXT_COMPONENT_LOWER_FRAME_POSITION;
elseif ~strcmpi(which_half, 'upper')
    return;
end

set(handles.grouping_labels(i), 'Position', label_position);
set(handles.grouping_boxes(i), 'Position', textfield_position);

label_position = label_position + shift;
textfield_position = textfield_position + shift;

set(handles.thickness_labels(i), 'Position', label_position);
set(handles.thickness_boxes(i), 'Position', textfield_position);

label_position = label_position + shift;
textfield_position = textfield_position + shift;

set(handles.epsilon_labels(i), 'Position', label_position);
set(handles.epsilon_boxes(i), 'Position', textfield_position);

label_position = label_position + shift;
textfield_position = textfield_position + shift;

set(handles.tan_delta_labels(i), 'Position', label_position);
set(handles.tan_delta_boxes(i), 'Position', textfield_position);


% --- Executes on button press in ok_btn.
function ok_btn_Callback(hObject, eventdata, handles)
% hObject    handle to ok_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Ensure that each layer occurs in exactly one layer group
[handles warning_string] = check_layer_occurrences(handles);
if ~isempty(warning_string)
    set(handles.warning_text, 'String', ['Error: ' warning_string]);
    guidata(hObject,handles);
    return;
end

% Ensure that the sum of the thicknesses is approximately equal to the
% total thickness
total_thickness = get(handles.total_thickness_edit, 'String');
[handles.total_thickness num_layers] = calc_total_thickness(handles);
if abs(handles.total_thickness - str2num(total_thickness)) > 2*handles.THICKNESS_TOLERANCE_RATIO*num_layers*handles.total_thickness
    set(handles.warning_text, 'String', ...
        'Total thickness is not equal to the sum of the thicknesses!');
    guidata(hObject,handles);
    return;
end

% needed for the 'save' line:
num_layer_groups = handles.num_layer_groups;
num_layer_groups_string = handles.num_layer_groups_string;

groupings   = {};
thicknesses = {};
epsilons    = {};
tan_deltas  = {};

for i=1:num_layer_groups
    groupings{i}   = get(handles.grouping_boxes(i),  'String');
    thicknesses{i} = get(handles.thickness_boxes(i), 'String');
    epsilons{i}    = get(handles.epsilon_boxes(i),   'String');
    tan_deltas{i}  = get(handles.tan_delta_boxes(i), 'String');
end


% Save data to file
save layers   num_layer_groups num_layer_groups_string ...
    groupings thicknesses epsilons tan_deltas total_thickness

handles.num_layers  = num_layers;
handles.thicknesses = [];
handles.epsilons    = [];
handles.tan_deltas  = [];
for i = 1:num_layer_groups
    handles.thicknesses(i) = str2num(thicknesses{i});
    handles.epsilons(i)    = str2num(epsilons{i});
    handles.tan_deltas(i)  = str2num(tan_deltas{i});
end
% handles.total_thickness is already defined

handles.exitmode = 'ok';
guidata(hObject,handles);
uiresume(handles.figure1);


% --- Executes on button press in cancel_btn.
function cancel_btn_Callback(hObject, eventdata, handles)
% hObject    handle to cancel_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.exitmode = 'cancel';
guidata(hObject,handles);
uiresume(handles.figure1);


function handles = create_frame_component_group(handles, i)
% Creates the labels and edit boxes for the ith group of layers

handles.grouping_labels(i)   = create_label( ['List of layer numbers in Group ' num2str(i) ]);
handles.grouping_boxes(i)    = create_textfield;

handles.thickness_labels(i)   = create_label( ['Thickness of each layer in Group ' num2str(i) ]);
handles.thickness_boxes(i)    = create_textfield;

handles.epsilon_labels(i)   = create_label( ['Epsilon of layers in Group ' num2str(i) ]);
handles.epsilon_boxes(i)    = create_textfield;
set(handles.epsilon_labels(i), 'TooltipString', 'Relative Permittivity');
set(handles.epsilon_boxes(i) , 'TooltipString', 'Relative Permittivity');

handles.tan_delta_labels(i) = create_label( ['Loss tangent of layers in Group ' num2str(i) ]);
handles.tan_delta_boxes(i)  = create_textfield;
set(handles.tan_delta_labels(i), 'TooltipString', 'Dielectric loss tangent');
set(handles.tan_delta_boxes(i) , 'TooltipString', 'Dielectric loss tangent');


function h = create_label(str, varargin)
% Creates an invisible label with the specified string as its text
% Optional second argument is the TooltipString

h = uicontrol('Style', 'text', 'Visible', 'off', 'Units', 'pixels', 'String', str, 'HorizontalAlignment', 'left');
if nargin >= 2
    set(h, 'TooltipString', varargin{1});
end


function h = create_textfield(varargin)
% Creates an invisible text field
% One optional argument is the string in the box

h = uicontrol('Style', 'edit', 'Visible', 'off', 'Units', 'pixels');
if nargin
    set(h, 'String', varargin{1});
end

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(h,'BackgroundColor','white');
else
    set(h,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


function handles = set_component_group_visibility(handles, i, vis)
set(handles.grouping_labels(i)   , 'Visible', vis);
set(handles.grouping_boxes(i)    , 'Visible', vis);
set(handles.thickness_labels(i)  , 'Visible', vis);
set(handles.thickness_boxes(i)   , 'Visible', vis);
set(handles.epsilon_labels(i)    , 'Visible', vis);
set(handles.epsilon_boxes(i)     , 'Visible', vis);
set(handles.tan_delta_labels(i)  , 'Visible', vis);
set(handles.tan_delta_boxes(i)   , 'Visible', vis);


function [thickness, num_layers] = calc_total_thickness(handles)
thicknesses = [];
for i=1:handles.num_layer_groups
    layers_in_group = str2num(get(handles.grouping_boxes(i),  'String'));
    thicknesses(layers_in_group) = str2num(get(handles.thickness_boxes(i), 'String'));
end
thickness = sum(thicknesses);
num_layers = length(thicknesses);


function [handles, warning_string] = check_layer_occurrences(handles)
% CHECK_LAYER_OCCURRENCES returns a string warning of multiple occurrences
% of layers or missing layers in layer groupings.  A maximum of one error
% is returned, and duplicate layers take precedence over missing layers.
% CHECK_LAYER_OCCURRENCES also updates handles.groupings.


layers_present = [];
parent_group = []; % Tells which group in which a particular layer occurs
handles.groupings = {};

for i=1:handles.num_layer_groups
    handles.groupings{i} = str2num(get(handles.grouping_boxes(i),  'String'));

    % Check for duplicate layers
    isect = intersect(handles.groupings{i}, layers_present);
    if isempty(isect)
        layers_present = union(handles.groupings{i}, layers_present);
        parent_group(handles.groupings{i}) = i;
    else
        warning_string = ['Layer ' num2str(isect(1)) ' occurs in groups ' num2str(parent_group(isect(1))) ' and ' num2str(i) '.'];
        return;
    end
end

% Check for missing layers
missing_layers = setdiff(1:layers_present(end), layers_present);
if isempty(missing_layers)
    warning_string = '';
else
    warning_string = ['Layer ' num2str(missing_layers(1)) ' is not in any group.'];
end

        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Code that does very little besides keeping MATLAB from complaining:   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --- Executes during object creation, after setting all properties.
function editbox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to the edit box (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function  total_thickness_edit_CreateFcn(hObject, eventdata, handles)
editbox_CreateFcn(hObject,eventdata,handles);

function num_layer_groups_edit_CreateFcn(hObject, eventdata, handles)
editbox_CreateFcn(hObject,eventdata,handles);

function  total_thickness_edit_Callback(hObject, eventdata, handles)
% hObject    handle to total_thickness_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
