function [t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Edges, PortIndexes] = metal(P, t, Faces, FacesNontrivial, AT, Edges, Port);
%METAL - handler for the metal structure
%
% Syntax:
%   [t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Edges, PortIndexes] = metal(P, t,
%   Faces, FacesNontrivial, AT, Edges, Port);
%
% Description:
%   METAL is a "wrapping" function for the metal structure that calls two
%   core functions EDGES and PORT. See help to these functions for the list
%   of arguments. 
%
%   Along with this, dielectric edge numbering is changed: the dielectric
%   edges that coincide with the metal edges are put up front. This
%   operation has almost no effect on the solution though. 
%   
%   The METAL function also searches for the "embedded" metal faces that
%   have the dielectric on both sides. The corresponding dielectric faces 
%   are marked as "nontrivial" and will be used later on in order to to
%   separate the basis functions on both sides of the embedded meta sheet. 
%   
%   MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
%   Authors: A. Marut, S. Makarov

if ~isempty(t)
    t       = sortrows(t')';
    t       = unique(t', 'rows')'; 
    t(4,:)  = 1;
    [Edgesm, Edgesb, Tp, Tm, Vp, Vm]    = edges(P, t);
    %   Put dielectric faces that coincide with the embedded metal faces to the nontrivial cathegory    
    tt              = t(1:3, :);     
    Index           = ismember(Faces', tt', 'rows');        %   indicator into all faces that coincide with the metal faces (1 or 0)
    Index           = find(Index >0);                       %   index into all contact faces
    Index           = union([1:FacesNontrivial], Index);    %   all nontrivial faces
    FacesNontrivial = length(Index);
    temp            = setdiff([1:size(Faces, 2)], Index);
    Faces           = [Faces(:, Index) Faces(:, temp)];
    AT              = [AT(:, Index)       AT(:, temp)];
    %   Put all metal edges up front
    Index           = ismember(Edges', Edgesm', 'rows');    %   indicator into all common edges (1 or 0)
    Index           = find(Index >0);                       %   index into all common edges
    temp            = setdiff([1:size(Edges, 2)], Index);
    Edges           = [Edges(:, Index)   Edges(:, temp)];
    %   Identify ports
    [t, Edgesm, Tp, Tm, Vp, Vm, PortIndexes] = port(P, t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Port);
        
else    
    Edgesm  = []; Edgesb = []; Tp = []; Tm = []; Vp = []; Vm = [];
    PortIndexes.excitation = [];    
    PortIndexes.termination= [];
end