function [t, Edgesm, Tp, Tm, Vp, Vm, PortIndexes] = port(P, t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Port);
%PORT - Port handler
%
% Syntax:
%   [Edgesm, Tp, Tm, Vp, Vm, PortIndexes] = port(P, t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Port);
%
% Description:
%   PORT identifies edge numbers for lumped ports (excitation, impedance,
%   and termination edges). The script also extends the RWG basis function
%   set by the basis functions corresponding to the lumped ports of finite
%   thickness. Only a vertical lumped port of finite thickness is
%   permitted, only between two border metal edges (e.g. a stripline or
%   microstrip connected through the port to the ground plane on the edge
%   of the ground plane)
%
% Input:
%   P - array of nodes for the base planar mesh (from STRUCT2D) 
%   t - array of triangles for the base planar mesh (from STRUCT2D)
%   Edgesm - array of inner edges
%   Edgesb - array of boundary (outer) edges
%   Tp - array of adjacent "plus"  triangles for every inner edge
%   Tm - array of adjacent "minus" triangles for every inner edge
%   Vp - array of opposite vertices for adjacent "plus"  triangles
%   Vm - array of opposite vertices for adjacent "minus" triangles
%   Port - array of selected edge centers
%
% Output:
%   t - array of triangles - fourth index updated
%   Edgesm - array of inner edges (extended by lumped ports with a finite gap)
%   Tp - array of adjacent "plus"  triangles for every inner edge (extended
%   by lumped ports with a finite gap)
%   Tm - array of adjacent "minus" triangles for every inner edge (extended
%   by lumped ports with a finite gap)
%   Vp - array of opposite vertices for adjacent "plus"  triangles
%   (extended by lumped ports with a finite gap)
%   Vm - array of opposite vertices for adjacent "minus" triangles
%   (extended by lumped ports with a finite gap)
%   PortIndexes - output structure with the fields:
%
%                 PortIndexes.excitation - indexes into excitation edges 
%                 (if any)
%                 PortIndexes.termination - indexes into termination edge(s) 
%                 (if any). These edges have 50 Ohm lumped load each.  
%
%   MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
%   Authors: A. Marut, S. Makarov

%   Unique array of edge centers
Ports = [Port.excitation Port.termination];

PortIndexes.excitation = [];
PortIndexes.termination= [];

if isempty(Ports)
    return;
end

%   Indexes into port arrays
Ports           = unique(Ports','rows')';

for m = 1:size(Ports, 2)
    PortArray.excitation(m) = 0;
    if ~isempty(Port.excitation)
        temp1           = repmat(Ports(:, m), [1 size(Port.excitation, 2)]);
        temp2           = Port.excitation - temp1;
        temp3           = sum(temp2.*temp2, 1);
        [dummy index]   = sort(temp3);
        if dummy(1)< 1e-12;
            PortArray.excitation(m) = index(1);
        end
    end
    PortArray.termination(m) = 0;
    if ~isempty(Port.termination)
        temp1           = repmat(Ports(:, m), [1 size(Port.termination, 2)]);
        temp2           = Port.termination - temp1;
        temp3           = sum(temp2.*temp2, 1);
        [dummy index]   = sort(temp3);
        if dummy(1)< 1e-12; 
            PortArray.termination(m) = index(1);
        end
    end        
end

t(4,:)          = 0;
CenterE(1,:)    = mean(reshape(P(1, Edgesm), size(Edgesm)));
CenterE(2,:)    = mean(reshape(P(2, Edgesm), size(Edgesm)));
CenterE(3,:)    = mean(reshape(P(3, Edgesm), size(Edgesm)));
CenterEb(1,:)   = mean(reshape(P(1, Edgesb), size(Edgesb)));
CenterEb(2,:)   = mean(reshape(P(2, Edgesb), size(Edgesb)));
CenterEb(3,:)   = mean(reshape(P(3, Edgesb), size(Edgesb)));        

%   Identify all ports and extend mesh if necessary
for m = 1:size(Ports, 2)
    temp1           = repmat(Ports(:, m), [1 size(CenterE, 2)]);
    temp2           = CenterE - temp1;
    temp3           = sum(temp2.*temp2, 1);
    [dummy index]   = sort(temp3);
    %   Inner edge found
    if dummy(1)< mean(temp3)/1e6         
        add             = index(1);
        t(4, Tp(add))   = 1;
        t(4, Tm(add))   = 2;              
        if (PortArray.excitation(m))
            PortIndexes.excitation = [PortIndexes.excitation      add];                                
        end
        if (PortArray.termination(m))
            PortIndexes.termination = [PortIndexes.termination    add];  
        end        
        % for a T-junction feed only
        if abs(dummy(1)-dummy(2))/(abs(dummy(1)) + 1e3*eps)<eps            
            add             = index(2);
            t(4, Tp(add))   = 1;
            t(4, Tm(add))   = 2;              
            if (PortArray.excitation(m))
                PortIndexes.excitation = [PortIndexes.excitation      add];                                
            end          
            if (PortArray.termination(m))
                PortIndexes.termination = [PortIndexes.termination    add];  
            end                    
        end
    %   Innner edge not found (lumped port of finite size)
    else
        temp1           = repmat(Ports(:,m), [1 size(CenterEb, 2)]);
        temp2           = CenterEb(1:2, :) - temp1(1:2, :);
        temp3           = sum(temp2.*temp2, 1);
        [dummy index]   = sort(temp3);                
        %   two edges
        FirstEdge   = index(1);
        SecondEdge  = index(2);
        if CenterEb(3, index(1))>CenterEb(3, index(2))
            FirstEdge   = index(2);
            SecondEdge  = index(1);
        end
        %   add port edge elements
        Tp_ = find(sum((t(1:3, :)==Edgesb(1, FirstEdge))+(t(1:3, :)==Edgesb(2, FirstEdge)),1)==2);
        Tm_ = find(sum((t(1:3, :)==Edgesb(1, SecondEdge))+(t(1:3, :)==Edgesb(2, SecondEdge)),1)==2);
        Vp_ = setdiff([1 2 3],[find(t(1:3, Tp_)==Edgesb(1, FirstEdge)) find(t(1:3, Tp_)==Edgesb(2, FirstEdge))]);
        Vm_ = setdiff([1 2 3],[find(t(1:3, Tm_)==Edgesb(1, FirstEdge)) find(t(1:3, Tm_)==Edgesb(2, FirstEdge))]);     
        Edgesm      = [Edgesm Edgesb(:, FirstEdge)];
        Tp          = [Tp Tp_];
        Tm          = [Tm Tm_];
        Vp          = [Vp Vp_];
        Vm          = [Vm Vm_];        
        add             = length(Edgesm);
        t(4, Tp(add))   = 1;
        t(4, Tm(add))   = 2;              
        if (PortArray.excitation(m))
            PortIndexes.excitation = [PortIndexes.excitation      add];                                
        end
        if (PortArray.termination(m))
            PortIndexes.termination = [PortIndexes.termination    add];  
        end
    end
end

if ~isempty(PortIndexes.excitation)     PortIndexes.excitation  = sort(PortIndexes.excitation); end;
if ~isempty(PortIndexes.termination)    PortIndexes.termination = sort(PortIndexes.termination); end;