function [sel_T] = removetets(P0, t0, e0, TLayer1, tLayer1, numbers);
%REMOVETETS - Selects tetrahedra to be cut for every layer; triangles to be
%retained for every layer
%
% Syntax:
%   [sel_T sel_top sel_via] = removetets(P0,t0,e0,TLayer1,FLayer1,numbers);
%
% Description:
%   REMOVETETS calls TRISELECT and removes tetrahedra corresponding to the
%   selected triangles.
%
% Input:
%   P0 - array of nodes for the volumetric mesh - base layer 
%   t0 - array of triangles for the base planar mesh (from STRUCT2D)
%   TLayer1 - array of tetrahedra for the volumetric mesh - base layer 
%   FLayer1 - array of faces for the volumetric mesh - base layer 
%   numbers - array of layer numbers
%
% Output:
%   sel_T - cell array of index arrays of tetrahedra to be cut, one layer
%           per cell.  A value of {NaN} (i.e., a cell array containing the
%           value NaN) is currently used to indicate that the user closed
%           the window.
%
% See also: TRISELECT.

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, A. Marut


if length(numbers)==0
    str = ' nothing'; % shouldn't be called with numbers empty anyway
    sel_T = {};
    return;
elseif length(numbers)==1
    str = [ ' layer #' num2str(numbers)];
else
    str = [ ' layers ' num2str(numbers)];
end

%   Select tetrahedra
string          = strcat('Remove tets from ',str); 
Sel             = triselect(P0, t0, string);
if isnan(Sel)
    sel_T = {NaN};
    return
else
    sel_T           = [];
    if ~isempty(Sel)
        for m = 1:size(TLayer1, 2)
            CenterXY(:,m) =  sum(P0(1:2, TLayer1(1:4,m)),2)/4; % centroid of mth tetrahedron
        end
        for m = 1:length(Sel)    
            % Find indices of points inside the mth triangle
            ind     = inpolygon( CenterXY(1,:), CenterXY(2,:), P0(1, t0(:, Sel(m))), P0(2, t0(:, Sel(m))));        
            index   = find(ind==1);
            if index    
                sel_T = [sel_T index];  % If such points exist, add their indices to sel_T
            end
        end
    end
end

temp = {};
for n=numbers
    temp{end+1} = sel_T;
end
sel_T = temp;


