function varargout = triselect(varargin)
%TRISELECT - Interface for selecting simplices in a planar mesh
%
% Syntax:
%   Sel  = triselect(P, simp, Name, color);
%
% Description:
%   TRISELECT allows the user to select triangles or edges in a planar mesh
%
% Input:
%   P - array of nodes for the base planar mesh (from STRUCT2D)
%   simp - array of either 1- or 2-simplices (i.e., edges or triangles,
%       respectively) for the base planar mesh (from STRUCT2D).  If simp
%       has two rows, or if simp has three rows of which two are identical,
%       then the user selects edges.  If simp has three unique rows, then
%       the user selects triangles.
%       No two simplices may have the same centroid.
%   Name - string to display in the title of the current gui window
%   color - an RGB triple denoting the color of selected simplices
%
% Output:
%   Sel - array of indices (in simp) for selected simplices
%         If the user closes the window, then Sel is NaN.

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Author: A. Marut

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @triselect_OpeningFcn, ...
                   'gui_OutputFcn',  @triselect_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before triselect is made visible.
function triselect_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to triselect (see VARARGIN)

% Choose default command line output for triselect
UserData.selected_simplices = [];

% inputs are P, simp, Name, and color

triselect_call_err_msg = 'triselect usage: Sel = triselect(P, simp, Name [, color] )';
if length(varargin) ~= 3 && length(varargin) ~= 4
    errordlg(triselect_call_err_msg, 'Wrong number of arguments')
else
    p      = varargin{1};
    simp   = varargin{2};
    Name   = varargin{3};
    if length(varargin) == 4
        color  = varargin{4};
    else
        color  = [1 1 1];
    end
    
    UserData.p    = p;
    UserData.simplices = simp;
    UserData.color = color;
    set(gcf, 'Name', Name);
end

% Initialize shared data

% Start out by selecting simplices in a polygon
UserData.mode = 'poly';
UserData.zoom = 'off';
set(handles.rb_mode_tri,  'Value', 0);
set(handles.rb_mode_poly, 'Value', 1);

% initially the polygon has no vertices and no edges
UserData.poly_vertices = [];
UserData.poly_edges    = [];

UserData.X    = p(1,:);     % row of x coordinates of vertices
UserData.Y    = p(2,:);     % row of y coordinates of vertices
if size(simp,1) > 3
    simp = simp(1:3,:);     % eliminate the subdomain number  
end
if all(simp(2,:) == simp(3,:))
    simp = simp(1:2,:);     % if simplices are just edges, eliminate the third vertex
end

if size(simp, 1) == 2 && length(varargin) ~= 4
    UserData.color = [0 0 1];
end

UserData.simplices = simp;

hold on
UserData.th =   draw_simplices(p, simp);
view(0, 90);

set(UserData.th, 'HitTest', 'off'); % click goes thru to axes
set(UserData.th, 'ButtonDownFcn', @clicked_simplex);
set(handles.axes1, 'ButtonDownFcn', @clicked_axes);

% Update handles structure
guidata(hObject, handles);
set(handles.figure1, 'UserData', UserData);

U = get(handles.figure1, 'Units');
set(handles.figure1, 'Units', 'normalized', 'Position', [0.05 0.05 0.9 0.9], 'Units', U);

% UIWAIT makes triselect wait for user response (see UIRESUME)
uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = triselect_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if isempty(handles)
    varargout{1} = NaN;
    return
end

UserData = get(handles.figure1, 'UserData');
% Get default command line output from handles structure
varargout{1} = UserData.selected_simplices;
%   selected_simplices     the numbers of selected patches/lines
close gcf



function h = draw_simplices(P, simp, C)
% Draws simplices on the current axes.
% Inputs:
%   P:      2- or 3-by-M array of points
%   simp:   2- or 3-by-N array of simplices
% Output:
%   h:      1-by-N array of handles
% Misc.:
%   The simplices are colored gray if triangles or blue if line segments.
%   
%   DRAW_SIMPLICES sets the 'UserData' property of each graphics object to
%   the column index within SIMP of the corresponding simplex.  Thus,
%       get(h(i), 'UserData')
%   returns i for all i within range.

h = zeros(1,size(simp,2));

if size(P,1) == 2
    P(3,:) = 0;
end

if size(simp,1) == 2
    for i = 1:size(simp,2)
        h(i) = line( 'XData', P(1,simp(:,i)), 'YData', P(2,simp(:,i)), 'ZData', P(3,simp(:,i)), ...
            'Color', [0.6 0.6 0.6], 'UserData', i);
    end
else
    for i = 1:size(simp,2)
        h(i) = patch('XData', P(1,simp(:,i)), 'YData', P(2,simp(:,i)), 'ZData', P(3,simp(:,i)), ...
            'FaceColor', [0.6 0.6 0.6], 'UserData', i);
    end
end

axis equal;
    


function clicked_axes(obj, eventdata)
% obj        handle to the axes
% eventdata  reserved - to be defined in a future version of MATLAB

UserData = get(gcf, 'UserData');
handles  = guihandles(obj);
cp = get(handles.axes1, 'CurrentPoint');
current_point = cp(1, 1:2);

if ~strcmp(UserData.zoom, 'off')
    xlim = get(handles.axes1, 'XLim');
    xrange = xlim(2) - xlim(1);
    ylim = get(handles.axes1, 'YLim');
    yrange = ylim(2) - ylim(1);
    xc = current_point(1);
    yc = current_point(2);
    if strcmp(UserData.zoom, 'in')
        zfactor = 0.25;
        set(handles.zoom_in_btn, 'Value', 0);
    else
        zfactor = 4;
        set(handles.zoom_out_btn, 'Value', 0);
    end
    xmin = xc - zfactor*xrange/2;
    xmax = xc + zfactor*xrange/2;
    ymin = yc - zfactor*yrange/2;
    ymax = yc + zfactor*yrange/2;
    set(handles.axes1, 'XLim', [xmin xmax]);
    set(handles.axes1, 'YLim', [ymin ymax]);
    if strcmp(UserData.mode, 'individual')
        set(UserData.th, 'HitTest', 'on')
    end
    UserData.zoom = 'off';
elseif strcmp(UserData.mode, 'poly')
    UserData = add_poly_point(UserData, current_point);
end
set(gcf, 'UserData', UserData);
guidata(obj, handles);



function clicked_simplex(obj, eventdata)
% Callback executed when the user clicks a simplex's graphics object
%  obj          The graphics object clicked
%  eventdata    Not used; required argument for all callbacks

UserData = get(gcf, 'UserData');
simp_number = get(obj, 'UserData');

if any(UserData.selected_simplices == simp_number)  % the simplex is already selected
    UserData = simp_deselect(UserData, simp_number);
else
    UserData = simp_select(UserData, simp_number);
end

set(gcf, 'UserData', UserData);



function UserData = simp_select(UserData, simp_number)
% UserData      the data structure with all necessary information, except: 
% simp_number   the index in UserData.simplices of the simplex to select

UserData.selected_simplices = union(UserData.selected_simplices, simp_number);
if strcmp(get(UserData.th(simp_number), 'Type'), 'line')
    set(UserData.th(simp_number), 'Color', UserData.color, 'LineWidth', 2);
else
    set(UserData.th(simp_number), 'FaceColor', UserData.color, 'EdgeColor', [0 0 1], 'LineWidth', 2);
end



function UserData = simp_deselect(UserData, simp_number)
% UserData      the data structure with all necessary information, except: 
% simp_number   the index in UserData.simplices of the simplex to select

k = find(UserData.selected_simplices == simp_number);
UserData.selected_simplices(k) = [];
if strcmp(get(UserData.th(simp_number), 'Type'), 'line')
    set(UserData.th(simp_number), 'Color', [0.6 0.6 0.6], 'LineWidth', 0.5);
else
    set(UserData.th(simp_number), 'FaceColor', [0.6 0.6 0.6], 'EdgeColor', 'k', 'LineWidth', 0.5);
end


    
function UserData = add_poly_point(UserData, current_point)
% UserData          the data structure with all necessary information, except: 
% current_point     a vector with the xy coordinates of a mouse click

% add the point to poly_vertices, and if it's in the triangle (but not
% line segment) with the first vertex, then finish the polygon.

xi = current_point(1);
yi = current_point(2);

% if the first point and this point belong to the same triangle, close the polygon.
if length(UserData.poly_vertices)
    x0 = UserData.poly_vertices(1,1);
    y0 = UserData.poly_vertices(2,1);

    if size(UserData.simplices, 1) == 3
        FirstTriangleNumber = tsearch(UserData.X, UserData.Y, UserData.simplices', x0, y0);
        TriangleNumber = tsearch(UserData.X, UserData.Y, UserData.simplices', xi, yi);
        if TriangleNumber == FirstTriangleNumber
            UserData.poly_vertices = [UserData.poly_vertices [xi;yi]];
            UserData = close_polygon(UserData);
            return;
        end
    end
    % draw a line and add current_point to the polygon
    UserData.poly_edges(end+1) = line([UserData.poly_vertices(1,end) ; xi], [UserData.poly_vertices(2,end) ; yi], 'HitTest', 'off');
    UserData.poly_vertices = [UserData.poly_vertices [xi;yi]];
else
    % just add current point to the polygon
    UserData.poly_vertices = [xi;yi];
end



function UserData = close_polygon(UserData)
% UserData           the data structure with all necessary information

if length(UserData.poly_vertices) > 2
    for simplex_number = 1:size(UserData.simplices, 2)
        simplex = UserData.simplices(:,simplex_number);
        centroid_x = mean(UserData.p(1, simplex));  % x coordinate of centroid
        centroid_y = mean(UserData.p(2, simplex));  % y coordinate of centroid
        if inpolygon(centroid_x, centroid_y, UserData.poly_vertices(1,:), UserData.poly_vertices(2,:))
            UserData = simp_select(UserData, simplex_number);
        end
    end
end
UserData = cancel_polygon(UserData);


% --- Executes on button press in done_button.
function done_button_Callback(hObject, eventdata, handles)
% hObject    handle to done_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

UserData = get(gcf, 'UserData');

UserData = close_polygon(UserData);
set(gcf, 'UserData', UserData);
uiresume(handles.figure1);


% --- Executes on button press in rb_mode_tri.
function rb_mode_tri_Callback(hObject, eventdata, handles)
% hObject    handle to rb_mode_tri (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.rb_mode_tri,  'Value', 1);
set(handles.rb_mode_poly, 'Value', 0);
UserData = get(gcf, 'UserData');
UserData.mode = 'individual';
set(UserData.th, 'HitTest', 'on');
% complete the current polygon
UserData = close_polygon(UserData);
guidata(hObject, handles);
set(gcf, 'UserData', UserData);



% --- Executes on button press in rb_mode_poly.
function rb_mode_poly_Callback(hObject, eventdata, handles)
% hObject    handle to rb_mode_poly (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.rb_mode_poly, 'Value', 1);
set(handles.rb_mode_tri,  'Value', 0);
UserData = get(gcf, 'UserData');
UserData.mode = 'poly';
set(UserData.th, 'HitTest', 'off');
UserData = close_polygon(UserData);
% start a new polygon
guidata(hObject, handles);
set(gcf, 'UserData', UserData);



% --- Executes on button press in select_all_btn.
function select_all_btn_Callback(hObject, eventdata, handles)
% hObject    handle to select_all_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

UserData = get(gcf, 'UserData');
UserData = cancel_polygon(UserData);
UserData.selected_simplices = 1:length(UserData.simplices);

if size(UserData.simplices, 1) == 2
    set(UserData.th, 'Color', UserData.color, 'LineWidth', 2);
else
    set(UserData.th, 'FaceColor', UserData.color, 'EdgeColor', [0 0 1], 'LineWidth', 2);
end

set(gcf, 'UserData', UserData');



% --- Executes on button press in cancel_polygon_btn.
function cancel_polygon_btn_Callback(hObject, eventdata, handles)
% hObject    handle to cancel_polygon_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

UserData = get(gcf, 'UserData');
UserData = cancel_polygon(UserData);
set(gcf, 'UserData', UserData);



function UserData = cancel_polygon(UserData)
delete(UserData.poly_edges); % remove all the lines
UserData.poly_edges = [];
UserData.poly_vertices = [];



% --- Executes on button press in zoom_in_btn.
function zoom_in_btn_Callback(hObject, eventdata, handles)
% hObject    handle to zoom_in_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
UserData = get(gcf, 'UserData');
UserData.zoom = 'in';
set(UserData.th, 'HitTest', 'off');
set(gcf, 'UserData', UserData);


% --- Executes on button press in zoom_out_btn.
function zoom_out_btn_Callback(hObject, eventdata, handles)
% hObject    handle to zoom_out_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
UserData = get(gcf, 'UserData');
UserData.zoom = 'out';
set(UserData.th, 'HitTest', 'off');
set(gcf, 'UserData', UserData);


% --- Executes on button press in close_poly_btn.
function close_poly_btn_Callback(hObject, eventdata, handles)
% hObject    handle to close_poly_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
UserData = get(gcf, 'UserData');
UserData = close_polygon(UserData);
guidata(hObject, handles);
set(gcf, 'UserData', UserData);
