function [P, T, t, relative_epsilons, tan_deltas, Port] = volumetric(layerdata, P, t);
%VOLUMETRIC - Creates the final mesh by combining meshes for every layer
%
% Syntax:
%   [P, T, t, relative_epsilons, tan_deltas] = volumetric(layerdata, P, t)
%
% Description:
%   This function uses Delaunay tesselation delaunayn to create 
%   the first (base) layer. All other layer meshes are derived 
%   from that mesh by cutting out unnecessary tetrahedra. Finally,
%   all layer meshes are combined with each other. 
%
% Input:
%   layerdata - structure array (from LAYERS)
%   P - array of nodes for the planar mesh (from STRUCT2D)
%   t - array of planar triangles (from STRUCT2D)
%
% Output:
%   P - array of nodes for the volumetric mesh 
%   T - array of tetrahedra for the volumetric mesh 
%   t - array of metal faces
%   relative_epsilons - array of permittivities of each tetrahedron
%   tan_deltas - array of dielectric loss tangents of each tetrahedron

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Kulkarni, A. Marut, S. Makarov

%   LAYERS
%   Number of intermediate layers
NL          = layerdata.num_layers;        
%   Z values of the vertices above each layer
z_values    = cumsum(layerdata.thicknesses);
%   Thickness (total)
h           = z_values(end);    

%   PLANAR MESH
t0          = t(1:3,:); 
P0          = P;

%   BASE LAYER
[PLayer1, TLayer1, tLayer1, t0, e0]= addlayer(P, t0, z_values(1));  
P0                                 = [P PLayer1];
t = [];
T = [];
relative_epsilons       = [];
tan_deltas              = [];
Port.excitation         = [];
Port.termination        = [];

%  VOLUMETRIC MESH FOR ALL LAYERS
P_cell = cell(1,NL);
T_cell = cell(1,NL);
t_cell = cell(1,NL);
eps_cell = cell(1, NL);
tan_delta_cell = cell(1, NL);

for i = 1:layerdata.num_layer_groups
    g = layerdata.groupings{i};                                             % g is a group of layers
    for m = g                                                               % m is a particular layer
        PLayerm         = PLayer1;                                          % layer nodes from base layer
        PLayerm(3,:)    = z_values(m);                                      % with a value of z corresponding to layer m
        P_cell{m}       = PLayerm;                                          % add these new nodes to the set of all nodes
    end
    
    % Select triangles four times, and after each time, check whether the
    % user closed the window.
    % Better coding style would separate the main code from the code to
    % check for the closing of the window, but the intuitive way to do
    % that is with an error, which should be used only in extraordinary
    % circumstances.
    
    % Triangle selection 1
    [sel_T]  = removetets(P0, t0, e0, TLayer1, tLayer1, g);     % select tetrahedra to be cut

    % Window check 1
    if ~isempty(sel_T) && any(isnan(sel_T{1}))
        P = NaN;
        T = NaN;
        return
    end

    % Triangle selection 2
    [sel_tb] = addbot(P0, t0, tLayer1, g);                      % select bottom metal faces for a group
    
    % Window check 2
    if ~isempty(sel_tb) && any(isnan(sel_tb{1}))
        P = NaN;
        T = NaN;
        return
    end
        
    % Triangle selection 3
    [sel_tv] = addvia(P0, e0, tLayer1, g);                      % select via metal faces for a group
    
    % Window check 3
    if ~isempty(sel_tv) && any(isnan(sel_tv{1}))
        P = NaN;
        T = NaN;
        return
    end
    
    % Port selection 4 - excitation
    if min(g) ~= 1 
        z = z_values(min(g)-1);
    else
        z = 0;
    end
    temp    = addfeed(P0, e0, tLayer1, z, g, 'Port 1: Select excitation edges (if any) of ');   % select feed metal edges 
    Port.excitation    = [Port.excitation temp];
    % Window check 4
    if isnan(temp)
        P = NaN;
        T = NaN;
        return
    end
    
    
    % Port selection 5 - termination
    if min(g) ~= 1 
        z = z_values(min(g)-1);
    else
        z = 0;
    end
    temp    = addfeed(P0, e0, tLayer1, z, g, 'Port 2: Select 50 Ohm termination edges (if any) of ');   % select termination metal edges 
    Port.termination    = [Port.termination temp];
    % Window check 5
    if isnan(temp)
        P = NaN;
        T = NaN;
        return
    end
    
    % Triangle selection 6
    [sel_tt] = addtop(P0, t0, tLayer1, g);                      % select top metal faces for a group
    
    % Window check 6
    if ~isempty(sel_tt) && any(isnan(sel_tt{1}))
        P = NaN;
        T = NaN;
        return
    end

    new_epsilon = confirmremove;
    if isempty(new_epsilon)
        P = NaN;
        T = NaN;
        return
    end
   
    %   Middle layers
    for j = 1:length(layerdata.groupings{i})
        m = g(j);                                                           % m is a particular layer
        TLayer                  = TLayer1 + (m-1)*size(PLayer1, 2);         % add a constant (for this layer) to T
        tLayer                  = tLayer1 + (m-1)*size(PLayer1, 2);        
        % set the epsilon of all tets in the layer to that layer's default
        epsLayer = layerdata.epsilons(m) * ones(1, size(TLayer, 2));
        tanLayer = layerdata.tan_deltas(m) * ones(1, size(TLayer, 2));                
        if new_epsilon == 1
            TLayer(:, sel_T{j})     = [];                                   % remove selected tets
            epsLayer(sel_T{j})      = [];
            tanLayer(sel_T{j})      = [];
        else
            % change the epsilon of the tets
            epsLayer(sel_T{j})      = new_epsilon;
        end
        tLayer(:, sel_tv{j})   = [];          
        T_cell{m}               = TLayer;                                  % add remaining tets to T
        t_cell{m}               = tLayer;                                  % add remaining faces to t
        eps_cell{m}             = epsLayer;
        tan_delta_cell{m}       = tanLayer;
    end
    %   Top layer
    tLayer                 = tLayer1 + (m-1)*size(PLayer1, 2); 
    tLayer(:, sel_tt{j})   = [];
    t_cell{m} = [t_cell{m} tLayer];
    %   Bottom layer    
    m = g(1);
    tLayer                 = tLayer1 + (m-1)*size(PLayer1, 2); 
    tLayer(:, sel_tb{1})   = [];
    t_cell{m} = [t_cell{m} tLayer];
end

for i = 1:NL
    P = [P P_cell{i}];
    T = [T T_cell{i}];
    t = [t t_cell{i}];
    relative_epsilons = [relative_epsilons eps_cell{i}];
    tan_deltas = [tan_deltas tan_delta_cell{i}];
end
