function varargout = struct2d(varargin)
%STRUCT2D - Creates planar mesh using PDE toolbox
%
% Syntax:
%   struct2d
%
% Description:
%   STRUCT2D allows the user to define the planar geometry of the base
%   layer for the resonator to be studied.  The planar geometry consists of
%   the projection in the x-y plane of all borders between different
%   domains.  To define the geometry, the user may delimit up to a total of
%   eight rectangles and ellipses in addition to one polygon having up to
%   eight vertices. After delimiting the shapes to be included in the
%   geometry, the user presses the "View mesh" button to view a triangular
%   mesh.  If the mesh is satisfactory, the user presses the button labeled
%   "Accept mesh" to save the data and then the "Close" button to exit.
%   More detailed information on using STRUCT2D can be found in the
%   documentation.
%
%   When the user presses the "Accept mesh" button, STRUCT2D saves the
%   planar mesh to the file struct2d.mat.  This file contains two matrices:
%
%   P - 3-by-M array of nodes; P(1,I) is the x-coordinate of the Ith node,
%       P(2,I) is the y-coordinate, and P(3,I) is zero and represents the
%       z-coordinate of the node.
%   t - 4-by-N array of triangles; t(1:3,J) are the three indices in P of
%       the vertices of the Jth triangle.  In addition, t(4,J) is the
%       subdomain number of the Jth triangle.

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: A. Marut

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @struct2d_OpeningFcn, ...
                   'gui_OutputFcn',  @struct2d_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT



% --- Executes just before struct2d is made visible.
function struct2d_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to struct2d (see VARARGIN)

s = pwd;
cd ..;
pathadd(s,0);
cd(s);

% Choose default command line output for struct2d
handles.output = hObject;

if ispc
    editbox_bgcolor = 'white';
else
    editbox_bgcolor = get(0,'defaultUicontrolBackgroundColor');
end

handles.     INCLUDE = 1;
handles.          XC = 2;
handles.          YC = 3;
handles.      SQUARE = 4;
handles.      CIRCLE = 5;
handles.       WIDTH = 6;
handles.      HEIGHT = 7;
handles.      BORDER = 8;
handles.BORDER_WIDTH = 9;

handles.NUM_RECTS = 8;
handles.NUM_POLY_VERTICES = 8;

HORIZONTAL_ALIGNMENT = 'center';

handles.rect_inputs = zeros(handles.NUM_RECTS, 9);
handles.poly_inputs = zeros(2, handles.NUM_POLY_VERTICES);

re_tag_prefixes = {'include',  'xc',   'yc',   'square',   'circle',      'width', 'height', 'border',   'borderwidth'};
re_styles =       {'checkbox', 'edit', 'edit', 'checkbox', 'radiobutton', 'edit',  'edit',   'checkbox', 'edit'};

for row = 1:handles.NUM_RECTS;
    for col = 1:length(re_tag_prefixes)
        handles.rect_inputs(row, col) = uicontrol( ...
            'Style',    re_styles{col}, ...
            'FontSize', 8.0, ...
            'Tag',      strcat(re_tag_prefixes{col}, num2str(row)), ...
            'String',   '', ...
            'Callback', '', ...
            'HorizontalAlignment', HORIZONTAL_ALIGNMENT);
        if strcmp(re_styles{col}, 'edit')
            set(handles.rect_inputs(row, col), 'BackgroundColor', editbox_bgcolor);
        end
    end
    set(handles.rect_inputs(row, handles.SQUARE), 'Value', 1.0, 'Callback', strcat('struct2d(''shape_square_Callback'',gcbo,[],guidata(gcbo),', num2str(row), ')' ));
    set(handles.rect_inputs(row, handles.CIRCLE), 'Value', 0.0, 'Callback', strcat('struct2d(''shape_circle_Callback'',gcbo,[],guidata(gcbo),', num2str(row), ')' ));
end


for col=1:handles.NUM_POLY_VERTICES
    x = 63*col - 15;
    handles.poly_lbl_x(col)    = uicontrol( 'Style', 'text', 'FontSize', 8.0, 'String', ['x' num2str(col)], 'Callback', '');
    handles.poly_inputs(1,col) = uicontrol( 'Style', 'edit', 'FontSize', 8.0, 'Tag', ['x' num2str(col)], 'String', '', 'Callback', '', 'HorizontalAlignment', HORIZONTAL_ALIGNMENT, 'BackgroundColor', editbox_bgcolor);
    handles.poly_lbl_y(col)    = uicontrol( 'Style', 'text', 'FontSize', 8.0, 'String', [ 'y' num2str(col) ], 'Callback', '');
    handles.poly_inputs(2,col) = uicontrol( 'Style', 'edit', 'FontSize', 8.0, 'Tag', ['y' num2str(col)], 'String', '', 'Callback', '', 'HorizontalAlignment', HORIZONTAL_ALIGNMENT, 'BackgroundColor', editbox_bgcolor);
end

set(handles.edit_triangle_size, 'HorizontalAlignment', HORIZONTAL_ALIGNMENT);

if exist('dimensions.m','file')
    load_prev(handles);
end

% Update handles structure
guidata(hObject, handles);

set_position(gcf, [10 100 700 1], 'pixels'); % the height (4th element) gets set to the minimum value
p = get_position(gcf, 'normalized');
set_position(gcf, [(1-min([p(3:4); 1 1]))/2  p(3:4)], 'normalized');
figure1_ResizeFcn(gcf, [], handles);

% UIWAIT makes struct2d wait for user response (see UIRESUME)
uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = struct2d_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

s = pwd;
cd ..;
pathremove;
cd(s);


% Get default command line output from handles structure
if isempty(handles)
    % user pressed X button
else
    varargout{1} = handles.output;
    close gcf;
end


% --- Executes on button press in a square.
function shape_square_Callback(hObject, eventdata, handles, source)
% hObject    handle to the square (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% source     an integer specifying in which row this event originated

square = handles.rect_inputs(source, handles.SQUARE);
circle = handles.rect_inputs(source, handles.CIRCLE);
set(square, 'Value', get(square, 'Max'));
set(circle, 'Value', get(circle, 'Min'));
guidata(hObject, handles);


% --- Executes on button press in a circle.
function shape_circle_Callback(hObject, eventdata, handles, source)
% hObject    handle to the circle (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% source     an integer specifying in which row this event originated

circle = handles.rect_inputs(source, handles.CIRCLE);
square = handles.rect_inputs(source, handles.SQUARE);
set(circle, 'Value', get(circle, 'Max'));
set(square, 'Value', get(square, 'Min'));
guidata(hObject, handles);


% --- Executes on button press in cb_include_polygon.
function cb_include_polygon_Callback(hObject, eventdata, handles)
% hObject    handle to cb_include_polygon (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% This function does nothing.


% --- Executes during object creation, after setting all properties.
function edit_triangle_size_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit_triangle_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


function edit_triangle_size_Callback(hObject, eventdata, handles)
% hObject    handle to edit_triangle_size (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% This function does nothing.


% --- Executes on button press in btn_view_mesh.
function btn_view_mesh_Callback(hObject, eventdata, handles)
% hObject    handle to btn_view_mesh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[p e t] = make_mesh(handles);
f = figure;
pdemesh(p,e,t);
axis equal;


% --- Calculates the mesh from the data entered by the user
function [p,e,t] = make_mesh(handles)
gd = [];    % geometry description matrix (see DECSG)
POLYGON = 2;
RECTANGLE = 3;
ELLIPSE = 4;

for rect=1:handles.NUM_RECTS
    inputs = handles.rect_inputs(rect,:);
    if get(inputs(handles.INCLUDE), 'Value') == 1.0
        xc_vec = str2num(get(inputs(handles.XC),     'String'));
        yc_vec = str2num(get(inputs(handles.YC),     'String'));
        width  = str2num(get(inputs(handles.WIDTH),  'String'));
        height = str2num(get(inputs(handles.HEIGHT), 'String'));
        border_width = str2num(get(inputs(handles.BORDER_WIDTH), 'String'));
        for xc = xc_vec
            for yc = yc_vec
                if get(inputs(handles.SQUARE), 'Value') == 1.0
                    % draw a rectangle
                    xmin = xc - width/2;
                    xmax = xc + width/2;
                    ymin = yc - height/2;
                    ymax = yc + height/2;
                    gd   = [gd [RECTANGLE; 4; xmin; xmax; xmax; xmin; ymax; ymax; ymin; ymin]]; 
                    if get(inputs(handles.BORDER), 'Value') == 1.0 && border_width < width/2 && border_width < height/2
                        xmin = xmin + border_width;
                        xmax = xmax - border_width;
                        ymin = ymin + border_width;
                        ymax = ymax - border_width;
                        gd   = [gd [RECTANGLE; 4; xmin; xmax; xmax; xmin; ymax; ymax; ymin; ymin]]; 
                    end
                else
                    % draw an ellipse
                    gd   = [gd [ELLIPSE; xc; yc; width/2; height/2; 0; 0; 0; 0; 0]]; 
                    if get(inputs(handles.BORDER), 'Value') == 1.0 && border_width < width/2 && border_width < height/2
                        gd   = [gd [ELLIPSE; xc; yc; width/2-border_width; height/2-border_width; 0; 0; 0; 0; 0]]; 
                    end
                end
            end
        end
    end
end

poly_vertices = [];
if get(handles.cb_include_polygon, 'Value') == 1.0
    for col=1:8
        xs = get(handles.poly_inputs(1,col),'String');
        ys = get(handles.poly_inputs(2,col),'String');
        if isempty(xs) || isempty(ys)
            break
        end
        x = str2num(xs);
        y = str2num(ys);
        poly_vertices = [poly_vertices ; [x y]];
    end
    n = size(poly_vertices, 1);
    if n > 2
        polygon = [ 2 ; n ; poly_vertices(:,1) ; poly_vertices(:,2) ];
        if n == 3
            polygon = [polygon ; 0 ; 0];
        elseif n > 4
            gd = [ gd ; zeros(2*n-8, size(gd,2)) ];
        end
        gd = [ gd polygon ];
    end
end

trisize = str2num(get(handles.edit_triangle_size, 'String'));
if isempty(trisize)
    trisize = inf;
end

dl = decsg(gd);
[p e t] = initmesh(dl, 'Hmax', trisize, 'Hgrad', 1.75);  % do not change Hgrad
p = p/1000; % mm


% --- Executes on button press in btn_accept_mesh.
function btn_accept_mesh_Callback(hObject, eventdata, handles)
% hObject    handle to btn_accept_mesh (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Save the user data
dimensions_m = fopen('codes/dimensions.m', 'wt');
fprintf(dimensions_m, '%%DIMENSIONS.M - Script used by STRUCT2D.M\n');
for i=1:handles.NUM_RECTS
    fprintf(dimensions_m, '   re_inc(%d) = logical(%d);\n', i, get(handles.rect_inputs(i,handles.INCLUDE),'Value'));
    fprintf(dimensions_m, '    re_xc{%d} = ''%s'';\n', i, get(handles.rect_inputs(i,handles.XC),'String'));
    fprintf(dimensions_m, '    re_yc{%d} = ''%s'';\n', i, get(handles.rect_inputs(i,handles.YC),'String'));
    if get(handles.rect_inputs(i,handles.SQUARE),'Value')
        fprintf(dimensions_m, ' re_shape{%d} = ''rectangle'';\n', i);
    else
        fprintf(dimensions_m, ' re_shape{%d} = ''ellipse'';\n', i);
    end
    fprintf(dimensions_m, ' re_width{%d} = ''%s'';\n', i, get(handles.rect_inputs(i,handles.WIDTH),'String'));
    fprintf(dimensions_m, 're_height{%d} = ''%s'';\n', i, get(handles.rect_inputs(i,handles.HEIGHT),'String'));
    fprintf(dimensions_m, 're_border(%d) = logical(%d);\n', i, get(handles.rect_inputs(i,handles.BORDER),'Value'));
    fprintf(dimensions_m, 're_border_width{%d} = ''%s'';\n', i, get(handles.rect_inputs(i,handles.BORDER_WIDTH),'String'));
end
fprintf(dimensions_m, '\n\ninclude_poly = logical(%d);\n\n', get(handles.cb_include_polygon,'Value'));
for i=1:handles.NUM_POLY_VERTICES
    fprintf(dimensions_m, 'poly_x{%d} = ''%s'';\n', i, get(handles.poly_inputs(1,i),'String'));
    fprintf(dimensions_m, 'poly_y{%d} = ''%s'';\n', i, get(handles.poly_inputs(2,i),'String'));
end
fprintf(dimensions_m, '\n\ntriangle_size = ''%s'';\n', get(handles.edit_triangle_size,'String'));
fclose(dimensions_m);

[p e t] = make_mesh(handles);
P           =   p;
P(3,:)      =   0;
t(1:3,:)    =   sort(t(1:3,:),1);

if  exist('struct3d.mat')
    delete('struct3d.mat');
end

save struct2d  P t;
figure;
viewer(P, t);
view(0, 90);


% --- Loads the contents of the fields from the last time struct2d was run
function load_prev(handles)
clear dimensions;
dimensions;
for i=1:handles.NUM_RECTS
    set(handles.rect_inputs(i,handles.INCLUDE),'Value',re_inc(i));
    set(handles.rect_inputs(i,handles.XC),'String',re_xc{i});
    set(handles.rect_inputs(i,handles.YC),'String',re_yc{i});
    set(handles.rect_inputs(i,handles.SQUARE),'Value',0);
    set(handles.rect_inputs(i,handles.CIRCLE),'Value',0);
    if strcmpi(re_shape{i},'rectangle')
        set(handles.rect_inputs(i,handles.SQUARE),'Value',1);
    else
        set(handles.rect_inputs(i,handles.CIRCLE),'Value',1);
    end
    set(handles.rect_inputs(i,handles.WIDTH),'String',re_width{i});
    set(handles.rect_inputs(i,handles.HEIGHT),'String',re_height{i});
    set(handles.rect_inputs(i,handles.BORDER),'Value',re_border(i));
    set(handles.rect_inputs(i,handles.BORDER_WIDTH),'String',re_border_width{i});
end
set(handles.cb_include_polygon,'Value',include_poly);
for i=1:handles.NUM_POLY_VERTICES
    set(handles.poly_inputs(1,i),'String',poly_x{i});
    set(handles.poly_inputs(2,i),'String',poly_y{i});
end
set(handles.edit_triangle_size,'String',triangle_size);
    

% --- Executes on button press in btn_close.
function btn_close_Callback(hObject, eventdata, handles)
% hObject    handle to btn_close (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiresume(handles.figure1);


% --- Executes on resizing the figure window.
function figure1_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to callback object (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

window_width = get_width(gcf, 'pixels');
window_height = get_height(gcf, 'pixels');

if window_width < 489
    window_width = 489;
    set_width(gcf, window_width, 'pixels');
end

if window_height < 420
    window_height = 420;
    set_height(gcf, window_height, 'pixels');
    p = get_position(gcf, 'normalized');
    if p(2)+p(4) > 1    % the top of the window is above the top of the screen
        set_y(gcf, 1-p(4), 'normalized');
        set_y(gcf, get_y(gcf, 'pixels')-16, 'pixels'); % make the title bar visible
    end
end

CHSP = 10;                      % Component (or Control) Horizontal SPacing
CVSP = (window_height-306)/19;  % Component (or Control) Vertical   SPacing
WIDTH_INCLUDE_LABEL = 35;
WIDTH_SHAPE_LABEL = 32;
SPACING_SHAPE_COLUMN = 1;
WIDTH_BORDER_LABEL = 34;
WIDTH_BORDER_WIDTH_LABEL = 66;  % Assumption: WIDTH_BORDER_WIDTH_LABEL <= WIDTH_TEXT_FIELD
WIDTH_CHECKBOX = 18;
WIDTH_RADIO_BUTTON = 18;
WIDTH_TEXT_FIELD = (window_width + 3 - (11*CHSP ...
    + max([WIDTH_INCLUDE_LABEL  WIDTH_CHECKBOX])  ...
    + max([WIDTH_SHAPE_LABEL    WIDTH_CHECKBOX+SPACING_SHAPE_COLUMN+WIDTH_RADIO_BUTTON]) ...
    + max([WIDTH_BORDER_LABEL   WIDTH_CHECKBOX]) + 2))/5;

TFW = WIDTH_TEXT_FIELD;  % Text Field Width


set_position(handles.lbl_title,        [                          0  18*CVSP+287   window_width      26], 'pixels');
set_position(handles.lbl_units,        [                          0  17*CVSP+274   window_width      18], 'pixels');

% Rectangle/ellipse frame:

set_position(handles.frame1,           [                   CHSP-  1   6*CVSP+ 98   window_width-15         10*CVSP+181], 'pixels');
set_position(handles.lbl_rect_ellipse, [                 2*CHSP-  2  16*CVSP+270   108                       15], 'pixels');
set_position(handles.lbl_include,      [                 2*CHSP      15*CVSP+262   WIDTH_INCLUDE_LABEL       15], 'pixels');
set_position(handles.lbl_xc,           [0.1*window_width+3*CHSP+  5  15*CVSP+262   12                        15], 'pixels');
set_position(handles.lbl_yc,           [0.3*window_width+4*CHSP- 36  15*CVSP+262   12                        15], 'pixels');
set_position(handles.lbl_shape,        [0.4*window_width+5*CHSP- 48  15*CVSP+262   WIDTH_SHAPE_LABEL         15], 'pixels');
set_position(handles.lbl_width,        [0.5*window_width+6*CHSP- 49  15*CVSP+262   28                        15], 'pixels');
set_position(handles.lbl_height,       [0.7*window_width+7*CHSP- 93  15*CVSP+262   31                        15], 'pixels');
set_position(handles.lbl_border,       [0.8*window_width+8*CHSP- 98  15*CVSP+262   WIDTH_BORDER_LABEL        15], 'pixels');
set_position(handles.lbl_border_width, [0.9*window_width+9*CHSP-122  15*CVSP+262   WIDTH_BORDER_WIDTH_LABEL  15], 'pixels');

for row = 1:handles.NUM_RECTS;
    y = 15*CVSP + 267 - (21+CVSP)*row;
    
    x = 2 * CHSP; % the x-coordinate of the label 'Include'
    x = x + max([0  (WIDTH_INCLUDE_LABEL - WIDTH_CHECKBOX)/2]);  % center the checkbox under the label
    
    set_position(handles.rect_inputs(row, handles.INCLUDE), [x y  WIDTH_CHECKBOX 21], 'pixels');
    
    x = x + WIDTH_CHECKBOX + max([0  (WIDTH_INCLUDE_LABEL - WIDTH_CHECKBOX)/2]) + CHSP;

    set_position(handles.rect_inputs(row, handles.XC), [x y TFW 21], 'pixels');
    
    x = x + TFW + CHSP;    
        
    set_position(handles.rect_inputs(row, handles.YC), [x y TFW 21], 'pixels');
    
    x = x + TFW + CHSP;
    x = x + max([0  (WIDTH_SHAPE_LABEL - (WIDTH_CHECKBOX+SPACING_SHAPE_COLUMN+WIDTH_RADIO_BUTTON))/2]);    
        
    set_position(handles.rect_inputs(row, handles.SQUARE), [x y  WIDTH_CHECKBOX 21], 'pixels');
    
    x = x + WIDTH_CHECKBOX + SPACING_SHAPE_COLUMN;
    
    set_position(handles.rect_inputs(row, handles.CIRCLE), [x y  WIDTH_RADIO_BUTTON 21], 'pixels');
    
    x = x + WIDTH_RADIO_BUTTON + max([0  (WIDTH_SHAPE_LABEL - (WIDTH_CHECKBOX+SPACING_SHAPE_COLUMN+WIDTH_RADIO_BUTTON))/2]) + CHSP;
    
    set_position(handles.rect_inputs(row, handles.WIDTH), [x y TFW 21], 'pixels');
    
    x = x + TFW + CHSP;    
        
    set_position(handles.rect_inputs(row, handles.HEIGHT), [x y TFW 21], 'pixels');
    
    x = x + TFW + CHSP + max([0  (WIDTH_BORDER_LABEL-WIDTH_CHECKBOX)/2]);

    set_position(handles.rect_inputs(row, handles.BORDER), [x y  WIDTH_CHECKBOX 21], 'pixels');
    
    x = x + WIDTH_CHECKBOX + max([0  (WIDTH_BORDER_LABEL-WIDTH_CHECKBOX)/2]) + CHSP;

    set_position(handles.rect_inputs(row, handles.BORDER_WIDTH), [x y TFW 21], 'pixels');
    
end

% Polygon frame:

TFW = (window_width + 2 - (4 + handles.NUM_POLY_VERTICES) * CHSP - max([WIDTH_CHECKBOX  WIDTH_INCLUDE_LABEL]))/handles.NUM_POLY_VERTICES;

set_position(handles.frame2,             [  CHSP-1  2*CVSP+25  window_width-2*CHSP+5  3*CVSP+69], 'pixels');
set_position(handles.lbl_polygons,       [2*CHSP-1  5*CVSP+85  46 15], 'pixels');
set_position(handles.lbl_include_poly,   [2*CHSP    4*CVSP+77  WIDTH_INCLUDE_LABEL 15], 'pixels');
set_position(handles.cb_include_polygon, [2*CHSP+9  4*CVSP+61  WIDTH_CHECKBOX 15], 'pixels');

for col=1:handles.NUM_POLY_VERTICES
    x = (TFW + CHSP)*(col - 1) + 3 * CHSP + max([WIDTH_CHECKBOX  WIDTH_INCLUDE_LABEL]);
    set_position(handles.poly_lbl_x(col),    [x  4*CVSP+77  TFW  15], 'pixels');
    set_position(handles.poly_inputs(1,col), [x  4*CVSP+58  TFW  21], 'pixels');
    set_position(handles.poly_lbl_y(col),    [x  3*CVSP+45  TFW  15], 'pixels');
    set_position(handles.poly_inputs(2,col), [x  3*CVSP+26  TFW  21], 'pixels');
end

% Things at the bottom of the window:
set_position(handles.lbl_tri_size,       [               CHSP-  4  CVSP    92                       22], 'pixels');
set_position(handles.edit_triangle_size, [             2*CHSP+ 86  CVSP-1  window_width-6*CHSP-358  26], 'pixels');
set_position(handles.btn_view_mesh,      [window_width-3*CHSP-272  CVSP-1  92                       26], 'pixels');
set_position(handles.btn_accept_mesh,    [window_width-2*CHSP-180  CVSP-1  92                       26], 'pixels');
set_position(handles.btn_close,          [window_width-  CHSP- 88  CVSP-1  92                       26], 'pixels');



% Returns the position [x y width height] of the UI object represented by a
% handle using the requested units of measurement
function p = get_position(h, units)
old_units = get(h, 'Units');
set(h, 'Units', units);
p = get(h, 'Position');
set(h, 'Units', old_units);


% Changes the position [x y width height] of the UI object represented by a
% handle using the requested units of measurement
function set_position(h, position, units);
old_units = get(h, 'Units');
set(h, 'Units', units);
set(h, 'Position', position);
set(h, 'Units', old_units);


% Returns the x-coordinate of the UI object represented by a
% handle using the requested units of measurement
function x = get_x(h, units)
p = get_position(h, units);
x = p(1);


% Changes the x-coordinate of the UI object represented by a
% handle using the requested units of measurement
function set_x(h, x, units);
p = get_position(h, units);
p(1) = x;
set_position(h, p, units);


% Returns the y-coordinate of the UI object represented by a
% handle using the requested units of measurement
function y = get_y(h, units)
p = get_position(h, units);
y = p(2);


% Changes the y-coordinate of the UI object represented by a
% handle using the requested units of measurement
function set_y(h, y, units);
p = get_position(h, units);
p(2) = y;
set_position(h, p, units);


% Returns the width of the UI object represented by a
% handle using the requested units of measurement
function width = get_width(h, units)
p = get_position(h, units);
width = p(3);


% Changes the width of the UI object represented by a
% handle using the requested units of measurement
function set_width(h, width, units);
p = get_position(h, units);
p(3) = width;
set_position(h, p, units);


% Returns the height of the UI object represented by a
% handle using the requested units of measurement
function height = get_height(h, units)
p = get_position(h, units);
height = p(4);


% Changes the x-coordinate of the UI object represented by a
% handle using the requested units of measurement
function set_height(h, height, units);
p = get_position(h, units);
p(4) = height;
set_position(h, p, units);
