%STRUCT3D - Creates a volumetric dielectric mesh      
%
% Syntax:
%   struct3d
%
% Description:
%   STRUCT3D allows the user to define the three-dimensional geometry of
%   the resonator/antenna to be studied.  This is done in two steps.
%
%   First, the user partitions the dielectric into groups of layers, which
%   may in turn be divided into individual layers to obtain better
%   resolution.  All layers within a group have the same thickness,
%   dielectric constant, dielectric loss tangent, and planar shape; the
%   user enters all of these except the last during this step.
%
%   In the second step, the user delimits the planar shape of each layer
%   group.  For each group, the mesh stored in struct2d.mat is displayed,
%   and the user selects the triangles that do not belong to this particular
%   group of layers.
%
%   After the user has defined the structure of the resonator, STRUCT3D
%   creates a tetrahedral mesh of the dielectric and displays a window
%   where the user may choose how to view the mesh.  The user may view the
%   outer faces or all of the edges; choosing either one causes it to be
%   displayed in a new window.
%
%   When STRUCT3D finishes, the following variables are in the current
%   workspace and in the main structure file, struct3d.mat:
%
%   P - array of nodes 
%   T - array of tetrahedra
%   Faces - array of dielectric faces
%   FacesBoundary - array of boundary dielectric faces
%   FacesNontrivial - number of nontrivial dielectric faces - placed in front
%   (either boundary faces or faces with nonzero dielectric contrast or faces 
%   that coincide with the embedded metal faces)
%   Edges - array of dielectric edges
%   AT - array of adjacent tetrahedra for every dielectric face
%   const - structure array of constants including dielectric data for
%       every tetrahedron
%
%   t - array of metal faces
%   Edgesm - array of inner metal edges
%   Edgesb - array of border metal edges
%   Tp, Tm - arrays of adjacent triangles to every (inner) metal edge
%   Vp, Vm - arrays of opposite vertexes for adjacent triangles 
%   CT - array of dielectric tetrahedra adjacent to metal faces
%   PortIndexes - array of port edges (excitation - 1 V each)
%   The port edges are found as the closest ones to the point array PORT. 
%   The number of port edges can be arbitrary.
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, S. Kulkarni, A. Marut

clear all

s = pwd;
cd ..;
pathadd(s, 0);
cd(s);

%   Check if the 3D mesh already exists
if  exist('struct3d.mat')
    load struct3d;
    view3d(P, T, FacesBoundary, t, const, 1);    
    return;
end

load struct2d;   
cd ('codes/');
layerdata = layers;
cd ..
if strcmp(layerdata.exitmode, 'close') || strcmp(layerdata.exitmode, 'cancel') ||...
        strcmp(layerdata.exitmode, 'error')
    s = pwd;
    cd ..;
    pathremove;
    cd(s);
    return;
end    

%   Create metal-dielectric mesh
layerdata.thicknesses = layerdata.thicknesses/1000;  % in mm
[P, T, t, relative_epsilons, tan_deltas, Port] = volumetric(layerdata, P, t);
if any(any(isnan(P))) || any(any(isnan(T)))
    s = pwd;
    cd ..;
    pathremove;
    cd(s);
    return
end

%   Define material constants 
[const] = emdata(P, T, layerdata, relative_epsilons, tan_deltas);

%   Find all faces of the tetrahedral mesh and associated tetrahedra (if any)
[T, Faces, FacesBoundary, FacesNontrivial, Edges, AT, ATE, const]   = face1(P, T, const);
    
%   Find parameters of the metal structure (if any)
[t, Edgesm, Edgesb, Tp, Tm, Vp, Vm, Edges, PortIndexes] = ...
                                    metal(P, t, Faces, FacesNontrivial, AT, Edges, Port);

if ~const.isdielectric 
    T = []; 
end;

%   Save the data
disp([ '|T| = '  num2str(size(T,2)) ]);
save('struct3d', 'P', 'T',                              ...
     'Faces', 'FacesBoundary', 'FacesNontrivial',       ...
     'Edges', 'AT', 'const', 't', 'Edgesm', 'Edgesb',   ...
     'Tp', 'Tm', 'Vp', 'Vm', 'PortIndexes');
view3d(P, T, FacesBoundary, t, const, 1);    