function GEOM = dielectric (P, t, T, Faces, FacesNontrivial, Edges, AT, const);
%DIELECTRIC - Creates edge basis functions and associated geometry parameters      
%
% Syntax:
%   GEOM = DIELECTRIC (P, T, Faces, FacesNontrivial, Edges, AT, CT, const);
%   
% Description:
%   DIELECTRIC creates edge-basis functions and associated geometry
%   parameters as described below
%
% Input:
%   P - array of nodes
%   T - array of tetrahedra
%   Faces - array of dielectric faces
%   FacesNontrivial - array of nontrivial dielectric faces
%       (either boundary faces or faces with nonzero dielectric contrast)
%   Edges - array of all dielectric edges
%   AT - array of adjacent tetrahedra for every dielectric face
%   const - a structure array of constants including dielectric data
%           for every tetrahedron
%   More detailed information on using DIELECTRIC can be found in the
%   documentation
%
% Output:
%   GEOM - structure array with the fields indicated below
%
% Parameter                Size                             Comment
%
% GEOM.P                  3-by-number of nodes         Array of node coordinates (same as P)
% GEOM.T                  4-by-TetrahedraTotal         Array of tetrahedra vertices (same as T) 
% GEOM.Faces              3-by-Facestotal              Array of vertices of the faces (same as Faces) 
% GEOM.AT                 2-by-Facestotal              Array of tetrahedra adjacent to each face (same as AT)
% GEOM.TetrahedraTotal    scalar                       Total number of tetrahedra in the structure
% GEOM.FacesTotal         scalar                       Total number of faces in the structure
% GEOM.EdgesTotal         scalar                       Total number of independent edge basis functions (BFs) in the structure
% GEOM.FacesNontrivial    scalar                       Total number of faces with non-zero dielectric contrast (placed in front of array Faces)
% GEOM.Edges              2-by_EdgesTotal              Array of edges (temporary)
% GEOM.EdgesFNI           1-by-FacesTotal              Array that specifies the total number of independent edge BFs for every face
% GEOM.EdgesFN            6-by-FacesTotal              Array of all independent edge BFs for a face (double edges may be present for embedded faces)
% GEOM.EdgesTNI           1-by-TetrahedraTotal         Array that specifies the total number of independent edge BFs for a tetrahedron
% GEOM.EdgesTN            6-by-TetrahedraTotal         Array of all independent edge BFs for a tetrahedron
% GEOM.BasisTC            3-by-6-by-TetrahedraTotal    Vector fields of all independent BFs for a tetrahedron
% GEOM.VolumeT            1-by-TetrahedraTotal         Tetrahedra volumes
% GEOM.CenterT            3-by-TetrahedraTotal         Geometry centers for tetrahedra
% GEOM.CenterF            3-by-FacesTotal              Geometry centers for faces
% GEOM.AreaF              1-by-FacesTotal              Face areas
% GEOM.Contrast_real      1-by-TetrahedraTotal         Dielectric contrast (epsilon_r - epsilon0)/epsilon_r for every tetrahedron
% GEOM.Contrast_imag      1-by-TetrahedraTotal         Imaginary component is present in case of complex epsilon_r
% GEOM.DiffContrast_real  6-by-FacesTotal              Differential dielectric constant of faces (double edges may be present for embedded faces)
% GEOM.DiffContrast_imag  6-by-FacesTotal              Imaginary component is present in case of complex epsilon_r (double edges may be present for embedded faces)
% GEOM.TTSS               1-by-TetrahedraTotal         Double potential self integrals (analytical inner integral + central-point outer integral)
% GEOM.FFSS               1-by-FacesNontrivial         Double potential self integrals (analytical inner integral + central-point outer integral)
% GEOM.SizeT              1-by-TetrahedraTotal         The distance from the center of each tetrahedron to the furthest vertex
% GEOM.SizeRatio          scalar                       Ratio of minimum wavelength to average tetrahedron size
% GEOM.MaxFreq            scalar                       Maximum frequency at which results are valid

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov

if isempty(T)
    GEOM = [];
    return;
else
    faces   = Faces(:,1:FacesNontrivial);
end

%   Integration domain (squared size)
R   = sqrt(5);

%   Gaussian formulas for all integrals
[coeffS, weightsS, IndexS]      = tri(3, 2);
[coeffT, weightsT, IndexT]      = tet(1, 1);

%   Find potential integrals
FF = int_face_face(P, faces, coeffS, weightsS, R^2);     
TT = int_tet_tet(P, T, coeffT, weightsT, 1e-3);  
if ~isempty(t)
    tF  = int_tri_face(P, P, t(1:3,:), faces, coeffS, weightsS, R^2);    
    Tt  = int_tet_tri(P, P, t, T, coeffS, weightsS, 1e-3);       
end

%   Create Gaussian integration points for non-singular integrals
GEOM.OrderF                 = IndexS;
GEOM.WF                     = weightsS'; 
GEOM.PointsF                = zeros(3, IndexS, FacesNontrivial);
for m = 1:FacesNontrivial
    Vertexes    = P(:,Faces(:,m));
    for n = 1:IndexS 
        G(:,n) = coeffS(1,n)*Vertexes(:,1) + ...
                 coeffS(2,n)*Vertexes(:,2) + ...
                 coeffS(3,n)*Vertexes(:,3);
    end
    GEOM.PointsF(:,:,m) = G;    
end
clear G; 
GEOM.OrderT                 = IndexT;
GEOM.WT                     = weightsT'; 
GEOM.PointsT                = zeros(3, IndexT, size(T,2));
for m = 1:size(T,2)
    Vertexes    = P(:,T(:,m));
    for n = 1:IndexT 
        G(:,n) = coeffT(1,n)*Vertexes(:,1) + ...
                 coeffT(2,n)*Vertexes(:,2) + ...
                 coeffT(3,n)*Vertexes(:,3) + ...
                 coeffT(4,n)*Vertexes(:,4);
    end
    GEOM.PointsT(:,:,m) = G;  
end

%   Find independent basis functions and create necessary arrays   
%   Create structure GEOM
str0                    = basis(P, T, Faces, Edges, AT, const);
GEOM.P                  = P;
GEOM.T                  = T;
GEOM.Faces              = Faces;
GEOM.AT                 = AT;
GEOM.TetrahedraTotal    = size(T, 2);
GEOM.FacesTotal         = size(Faces, 2);
GEOM.EdgesTotal         = size(str0.Edge, 2);

%   This is how it should be for the independent exe files
GEOM.BasisTC            = zeros(3, 6, GEOM.TetrahedraTotal);
for m = 1:size(T, 2)
    GEOM.BasisTC(:,:, m) = reshape(str0.BasisTC(:, m), [3, 6]);
end
GEOM.BasisTCn           = str0.BasisTC;

GEOM.Edges              = str0.Edge;
GEOM.FacesNontrivial    = FacesNontrivial;
GEOM.EdgesFNI           = str0.EdgesFNI;
GEOM.EdgesFN            = str0.EdgesFN;
GEOM.EdgesTNI           = str0.EdgesTNI;
GEOM.EdgesTN            = str0.EdgesTN;
GEOM.DiffContrast_real  = str0.DiffContrast_real;
GEOM.DiffContrast_imag  = str0.DiffContrast_imag;
GEOM.VolumeT            = str0.VolumeT;
GEOM.Contrast_real      = str0.Contrast_real;
GEOM.Contrast_imag      = str0.Contrast_imag;
GEOM.CenterT            = str0.CenterT;
GEOM.CenterF            = str0.CenterF;
GEOM.AreaF              = str0.AreaF;
GEOM.TTIS               = TT.IS;
GEOM.TTJS               = TT.JS;
GEOM.TTSS               = TT.SS;
GEOM.FFIS               = FF.IS;
GEOM.FFJS               = FF.JS;
GEOM.FFSS               = FF.SS;

GEOM.EdgesFN(4:6, :)              = 0;
GEOM.DiffContrast_real(4:6, :)    = 0;
GEOM.DiffContrast_imag(4:6, :)    = 0;

if ~isempty(t)
    GEOM.tFIS           = tF.IS;
    GEOM.tFJS           = tF.JS;
    GEOM.tFSS           = tF.SS;
    GEOM.TtIS           = Tt.IS;
    GEOM.TtJS           = Tt.JS;
    GEOM.TtRS           = Tt.RS;        
end

% Size of tetrahedra defined as distance from center to furthest vertex
X = GEOM.P(1,:);
Y = GEOM.P(2,:);
Z = GEOM.P(3,:);
VertexX = X(GEOM.T);
VertexY = Y(GEOM.T);
VertexZ = Z(GEOM.T);
CenterX = repmat(GEOM.CenterT(1,:), [4 1]);
CenterY = repmat(GEOM.CenterT(2,:), [4 1]);
CenterZ = repmat(GEOM.CenterT(3,:), [4 1]);
distances = sqrt((VertexX - CenterX).^2 + (VertexY - CenterY).^2 + (VertexZ - CenterZ).^2);

GEOM.SizeT = max(distances);
GEOM.SizeRatio = 5;
GEOM.MaxFreq = const.c / (GEOM.SizeRatio * mean(GEOM.SizeT) * max(sqrt(const.Epsilon_r / const.epsilon)));