function GEOM = f_basis(GEOM, geom, AT, FacesNontrivial, const);
%F_BASIS - Subdivides/modifies basis functions
%
% Syntax:
%   GEOM = f_basis(GEOM, AT, CT, FacesNontrivial, const);
% 
% Description:
%   F_BASIS returns a structure GEOM with modified basis functions (BFs)
%   close to the metal faces. In short:
%
%   i.  The BFs that are crossing the embedded metal faces are subdivided.
%   ii. The BF tangential component on the metal faces is essentially removed. 
%   iii.These "no-normal-component" basis functions are substituted 
%       in the D/eps term only
%
% Inputs:
%   GEOM - dielectric structure
%   geom - metal structure
%   AT   - array of tetrahedra adjacent to dielectric faces
%   FacesNontrivial - array of faces with nontrivial contrast
%   const - a structure array of constants including dielectric data
%           for every tetrahedron
%
% Outputs:
%   GEOM - modified structure GEOM
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu
% Authors: S. Makarov, S. Kulkarni

%   Find the embedded metal faces (ContactS) and other parameters
tt                      = geom.t(1:3, :);     
faces                   = GEOM.Faces(:, 1:FacesNontrivial);
Index                   = ismember(faces', tt', 'rows');        %   indicator into all contact faces (1 or 0)
Index                   = find(Index >0);                       %   index into all contact faces
ContactT = [];
ContactS = [];
ContactF = [];
for m = 1:length(Index)                                          %   loop over faces
    FaceNumber = Index(m);
    ContactT = [ContactT AT(2, FaceNumber)];
    ContactS = [ContactS FaceNumber];
    temp     = AT(1, FaceNumber);   
    if temp > 0
       ContactT = [ContactT temp];       
       ContactS = [ContactS FaceNumber];
       ContactF = [ContactF FaceNumber];
    end
end

%   Find all BFs for which these faces are embedded
%   Subdivide the BFs
%   Modify GEOM
if ~isempty(ContactF)
    ADD   = GEOM.EdgesTotal;
    for m = 1:GEOM.EdgesTotal                       %   loop over edges    
        [R1 C1] = find(faces(:,ContactF)==GEOM.Edges(1, m));
        [R2 C2] = find(faces(:,ContactF)==GEOM.Edges(2, m));
        F       = intersect(C1, C2);                % index into ContactF
        %   C may contain one or two faces
        if (length(F)==2)                           % only two inner faces        
            ADD = ADD + 1;                          % new BF
            Faces  = ContactF(F);
            Vector1 = GEOM.CenterF(:, Faces(1))    - GEOM.CenterF(:, Faces(2));  
            Vector2 = GEOM.P(:, GEOM.Edges(1, m))  - GEOM.P(:, GEOM.Edges(2, m));              
            Vector3 =(GEOM.P(:, GEOM.Edges(1, m))  + GEOM.P(:, GEOM.Edges(2, m)))/2;
            Vector4 = cross(Vector1, Vector2); 
            %   Tetrahedra
            [R C] = find(GEOM.EdgesTN == m);          % C - tet number, R - local            
            for n = 1:length(C)
                Vector5 = GEOM.CenterT(:,C(n)) - Vector3; 
                BF      = GEOM.BasisTC(:, R(n), C(n));
                DOT     = dot(Vector4, Vector5);
                %   Introduce new BF - new
                if DOT >0                           % one side  (left or right)/(top or bottom)                              
                    GEOM.EdgesTN(R(n), C(n)) = ADD;                     % new BF
                    for p = 1:2
                        Face = Faces(p);
                        if ~isempty(find(AT(:, Face) == C(n)))    % only one adjacent tet                            
                            IN          = dot(BF, GEOM.CenterF(:, Face)-GEOM.CenterT(:, C(n)));
                            Contrast    = const.Epsilon_r(C(n))*(1 - j*const.tan_delta(C(n)));
                            DiffContrast= (Contrast - const.epsilon)/Contrast;
                            RF          = find(GEOM.EdgesFN(:, Face)==m);   % local indexes
                            temp        = GEOM.EdgesFNI(Face) + 1;
                            GEOM.EdgesFNI(Face)      = temp;
                            GEOM.EdgesFN(temp, Face) = ADD;  % new BF
                            GEOM.DiffContrast_real(temp, Face) = sign(IN)*real(DiffContrast);
                            GEOM.DiffContrast_imag(temp, Face) = sign(IN)*imag(DiffContrast);                        
                        end                 
                    end                    
                else
                    for p = 1:2
                        Face = Faces(p);
                        if ~isempty(find(AT(:, Face) == C(n)))    % only one adjacent tet                            
                            IN          = dot(BF, GEOM.CenterF(:, Face)-GEOM.CenterT(:, C(n)));
                            Contrast    = const.Epsilon_r(C(n))*(1 - j*const.tan_delta(C(n)));
                            DiffContrast= (Contrast - const.epsilon)/Contrast;
                            RF          = find(GEOM.EdgesFN(:, Face)==m);   % local indexes
                            GEOM.DiffContrast_real(RF, Face) = sign(IN)*real(DiffContrast);
                            GEOM.DiffContrast_imag(RF, Face) = sign(IN)*imag(DiffContrast);
                        end
                    end                
                end                               
            end            
        end
    end 
    GEOM.EdgesTotal = ADD;  
    GEOM.Edges      = [];
end   
                
%   Introduce the boundary condition        
GEOM.BasisTCn   = GEOM.BasisTC;
par             = 0.9;
for m = 1:length(ContactS)
    FaceNumber  = ContactS(m);
    TetNumber   = ContactT(m);
    Vertexes    = GEOM.P(:, GEOM.Faces(:, FaceNumber));
    C1          = Vertexes(:,1) -Vertexes(:,2);
    C2          = Vertexes(:,1) -Vertexes(:,3);
    N           = cross(C1, C2);
    N           = N/norm(N);    
    for n = 1:6        
        NormalComp                          = dot(GEOM.BasisTC(:, n, TetNumber), N)*N;
        TangComp                            = GEOM.BasisTC(:, n, TetNumber) - NormalComp;
        GEOM.BasisTCn(:, n, TetNumber)      = par*NormalComp + (1 - par)*TangComp;
    end
end
