function [geom] = metal(P, t, Tp, Tm, Vp, Vm, Edgesm, const);
%METAL - Creates RWG basis functions and associated geometry parameters      
%
% Syntax:
%   geom = METAL (P, t, Tp, Tm, Vp, Vm, Edgesm);
%   
% Description:
%   METAL creates RWG functions and associated geometry
%   parameters as described below.
%
% Input:
%   P   - array of nodes
%   t   - array of triangles
%   Tp  - adjacent (plus) triangle for every inner metal edge
%   Tm  - adjacent (minus) triangle for every inner metal edge
%   Vp  - free vertex of adjacent (plus) triangle 
%   Vm  - free vertex of adjacent (minus) triangle 
%   Edgesm  - array of inner metal edges
%
% Output:
%   geom - structure array with the fields indicated below
%       geom.OrderM
%       geom.WM
%       geom.PointsM
%       geom.PointsMRho
%       geom.PointsFCRho
%       geom.P
%       geom.t
%       geom.TrianglesTotal
%       geom.EdgesTotal
%       geom.TriP
%       geom.TriM
%       geom.VerP
%       geom.VerM
%       geom.AreaF
%       geom.EdgeLength
%       geom.PointsFC
%       geom.ttIS
%       geom.ttJS
%       geom.ttSS
%       geom.ttRS
%       geom.SizeT
%       geom.SizeRatio
%       geom.MaxFreq

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov


if isempty(t)
    geom = [];
    return;
end

%   Integration domain - linear
    R   = sqrt(0.1);    % do not change!!!!
%   Gaussian formulas for all integrals
[coeffS, weightsS, IndexS]      = tri(7, 5);

%   Find potential integrals for metal faces
tt  = int_tri_tri(P, t, coeffS, weightsS, R^2);  

%   Create Gaussian integration points for triangles
geom.OrderM                 = IndexS;
geom.WM                     = weightsS'; 
geom.PointsM                = zeros(3, IndexS,    size(t, 2));
geom.PointsMRho             = zeros(3, IndexS, 3, size(t, 2));
for m = 1:size(t,2)
    Vertexes    = P(:,t(1:3,m));
    for n = 1:IndexS 
        G(:,n) = coeffS(1,n)*Vertexes(:,1) + ...
                 coeffS(2,n)*Vertexes(:,2) + ...
                 coeffS(3,n)*Vertexes(:,3);
    end
    geom.PointsM(:,:,m) = G;    
    geom.PointsMRho(:,:,1,m) = G - repmat(P(:,t(1,m)),[1, IndexS]);
    geom.PointsMRho(:,:,2,m) = G - repmat(P(:,t(2,m)),[1, IndexS]);
    geom.PointsMRho(:,:,3,m) = G - repmat(P(:,t(3,m)),[1, IndexS]);    
end

%   Create structure GEOM
geom.PointsFCRho        = zeros(3, 3, size(t,2));
geom.P                  = P;
geom.t                  = t;   
geom.TrianglesTotal     = size(t,2);
geom.EdgesTotal         = size(Tp,2);
geom.TriP               = Tp;
geom.TriM               = Tm;
geom.VerP               = Vp;
geom.VerM               = Vm;
geom.AreaF              = tt.AreaF;
temp                    = P(:,Edgesm(1,:))-P(:,Edgesm(2,:));
geom.EdgeLength         = sqrt(dot(temp,temp,1));
geom.PointsFC           = tt.PointsFC;
geom.PointsFCRho(:,1,:) = geom.PointsFC - P(:,t(1,:));
geom.PointsFCRho(:,2,:) = geom.PointsFC - P(:,t(2,:));
geom.PointsFCRho(:,3,:) = geom.PointsFC - P(:,t(3,:));
geom.ttIS               = tt.IS;
geom.ttJS               = tt.JS;
geom.ttSS               = tt.SS;
geom.ttRS               = tt.RS;
geom.R                  = R;

% Size of triangles defined as distance from center to furthest vertex
temp = t(1:3, :);
X = geom.P(1,:);
Y = geom.P(2,:);
Z = geom.P(3,:);
VertexX = X(temp);
VertexY = Y(temp);
VertexZ = Z(temp);
CenterX = repmat(geom.PointsFC(1,:), [3 1]);
CenterY = repmat(geom.PointsFC(2,:), [3 1]);
CenterZ = repmat(geom.PointsFC(3,:), [3 1]);
distances = sqrt((VertexX - CenterX).^2 + (VertexY - CenterY).^2 + (VertexZ - CenterZ).^2);

geom.SizeT = max(distances);
geom.SizeRatio = 8;
geom.MaxFreq = const.c / (geom.SizeRatio * mean(geom.SizeT));