function [coeff, weights, IndexT] = tet(arg1, arg2);

% Creates integration points and weights for tetrahedra 
% Syntax:
% [coeff, weights, IndexT] = tet(2) -   will do barycentric subdivision with
%                                       2*2*2 = 8 integration points
% [coeff, weights, IndexT] = tet(5,3) - will use Gaussian cubature of third 
%                                       order with five integration points
%
% Description:
% Uses Gaussian cubatures (with author's permission) from 
% http://www.cs.kuleuven.ac.be/~nines/research/ecf/ecf.html
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu

nargs = nargin; 

if  nargs < 1
    error('Requires at least 1 input');
end

if nargs == 1
%   Barycentric tetrahedra subdivision - coefficients for vertexes only
%   M - subdivision order (number of subtetrahedra is 8^M)
    M = arg1;	
    if M < 2 
        coeff(:,1) = [1/4 1/4 1/4 1/4]';
        weights(1) = 1; 
    end	
    if M == 2        
		p11 = [1.0 0.0 0.0 0.0]';
		p22 = [0.0 1.0 0.0 0.0]';
		p33 = [0.0 0.0 1.0 0.0]';
		p44 = [0.0 0.0 0.0 1.0]';
		p12 = [0.5 0.5 0.0 0.0]';
		p13 = [0.5 0.0 0.5 0.0]';
		p14 = [0.5 0.0 0.0 0.5]';
		p23 = [0.0 0.5 0.5 0.0]';
		p24 = [0.0 0.5 0.0 0.5]';
		p34 = [0.0 0.0 0.5 0.5]';
		coeff(:,1) = p11 + p12 + p13 + p14;
        coeff(:,2) = p22 + p12 + p23 + p24;
        coeff(:,3) = p33 + p13 + p23 + p34;
        coeff(:,4) = p44 + p14 + p24 + p34;        
        coeff(:,5) = p12 + p13 + p14 + p24;
        coeff(:,6) = p12 + p13 + p23 + p24;
        coeff(:,7) = p13 + p23 + p24 + p34;
        coeff(:,8) = p13 + p14 + p24 + p34;
        coeff       = coeff/4;
        weights     = ones(1,8)/8;
    end	 
    if M==3
        p11 = [1.0 0.0 0.0 0.0]';
        p22 = [0.0 1.0 0.0 0.0]';
        p33 = [0.0 0.0 1.0 0.0]';
        p44 = [0.0 0.0 0.0 1.0]';
        p12 = [0.5 0.5 0.0 0.0]';
        p13 = [0.5 0.0 0.5 0.0]';
        p14 = [0.5 0.0 0.0 0.5]';
        p23 = [0.0 0.5 0.5 0.0]';
        p24 = [0.0 0.5 0.0 0.5]';
        p34 = [0.0 0.0 0.5 0.5]';
        coeff1(:,1)      = p11 + p12 + p13 + p14;
        coeff1(:,2)      = p22 + p12 + p23 + p24;
        coeff1(:,3)      = p33 + p13 + p23 + p34;
        coeff1(:,4)      = p44 + p14 + p24 + p34;
        coeff1(:,5)      = p12 + p13 + p14 + p24;
        coeff1(:,6)      = p12 + p13 + p23 + p24;
        coeff1(:,7)      = p13 + p23 + p24 + p34;
        coeff1(:,8)      = p13 + p14 + p24 + p34;
        Factor(:,:,1)   = [p11 p12 p13 p14];
        Factor(:,:,2)   = [p22 p12 p23 p24];
        Factor(:,:,3)   = [p33 p13 p23 p34];
        Factor(:,:,4)   = [p44 p14 p24 p34];
        Factor(:,:,5)   = [p12 p13 p14 p24];
        Factor(:,:,6)   = [p12 p13 p23 p24];
        Factor(:,:,7)   = [p13 p23 p24 p34];
        Factor(:,:,8)   = [p13 p14 p24 p34];
        for m = 1:8
            for n = 1:8
                coeff(:,n+(m-1)*8) = squeeze(Factor(:,:,n)*coeff1(:,m));
            end
        end
        coeff       = coeff/4;
        weights     = ones(1,64)/64;
    end        
end

if nargs == 2
%   Gaussian cubature formulae
%   arg1 - number of integration points 
%   arg2 - order of the formula 

    %   Cools
    if arg1 == 1    %    1st order                
		coeff(:,1)  = [1/4  1/4  1/4  1/4]';
	    weights(1)  =  1;
    end        
    %   Cools
    if  (arg1==4) & (arg2==2)                       %  2nd order                
	    a1 = 0.58541020;
		b1 = 0.13819660;
		coeff(:,1)  = [a1  b1  b1  b1]';
		coeff(:,2)  = [b1  a1  b1  b1]';
		coeff(:,3)  = [b1  b1  a1  b1]';
        coeff(:,4)  = [b1  b1  b1  a1]';
	    weights(1)  =   0.25;
		weights(2)  =   0.25;
		weights(3)  =   0.25;
		weights(4)  =   0.25;    	
    end
    %   Cools
    if (arg1==10) & (arg2==2)                       %  2nd order
		coeff(:,1)      = [1/2  1/2  0   0]';
		coeff(:,2)      = [0    1/2  1/2 0]';
		coeff(:,3)      = [1/2  0    1/2 0]';
        coeff(:,4)      = [0    1/2  0  1/2]';
        coeff(:,5)      = [1/2   0   0  1/2]';
        coeff(:,6)      = [0     0  1/2 1/2]';
		coeff(:,7)      = [1  0  0   0]';
		coeff(:,8)      = [0  1  0   0]';
		coeff(:,9)      = [0  0  1   0]';
        coeff(:,10)     = [0  0  0   1]';        
        weights(1:6)    =  0.033333333333333;
	    weights(7:10)   = -0.0083333333333333;
        weights         = weights*6;
    end
    %   Cools
    if (arg1==5) & (arg2==3)                        %  3rd order                
	    a1 = 1/2;
		b1 = 1/6;
        coeff(:,1)  = [1/4  1/4  1/4  1/4]';
		coeff(:,2)  = [a1  b1  b1  b1]';
		coeff(:,3)  = [b1  a1  b1  b1]';
		coeff(:,4)  = [b1  b1  a1  b1]';
        coeff(:,5)  = [b1  b1  b1  a1]';
	    weights(1)  =   -4/5;
		weights(2)  =   9/20;
		weights(3)  =   9/20;
		weights(4)  =   9/20;
        weights(5)  =   9/20;      
    end        
    %   Cools
    if  (arg1==11) & (arg2==4)                      % 4th order                
	    a1 = 0.7857143;
		b1 = 0.0714286;
        a2 = 0.3994036;
        b2 = 0.1005964; 
        coeff(:,1)  = [1/4  1/4  1/4  1/4]';
		coeff(:,2)  = [a1  b1  b1  b1]';
		coeff(:,3)  = [b1  a1  b1  b1]';
		coeff(:,4)  = [b1  b1  a1  b1]';
        coeff(:,5)  = [b1  b1  b1  a1]';          
        coeff(:,6)  = [a2  a2  b2  b2]';        
	    coeff(:,7)  = [a2  b2  a2  b2]';    
        coeff(:,8)  = [a2  b2  b2  a2]';
        coeff(:,9)  = [b2  a2  a2  b2]';
        coeff(:,10) = [b2  a2  b2  a2]';
        coeff(:,11) = [b2  b2  a2  a2]';
	    weights(1)  = -0.01315556; 
        weights(2)  =  0.00762222; 
        weights(3)  =  0.00762222; 
        weights(4)  =  0.00762222; 
        weights(5)  =  0.00762222; 
        weights(6)  =  0.02488889; 
        weights(7)  =  0.02488889; 
        weights(8)  =  0.02488889; 
        weights(9)  =  0.02488889; 
        weights(10) =  0.02488889; 
        weights(11) =  0.02488889;
        weights     =  weights*6; 
    end          
    %   Cools
    if (arg1 == 14) & (arg2 == 5)                   %  5th order
        a1 = 0.72179425;
		b1 = 0.09273525;
        a2 = 0.06734224;
        b2 = 0.31088592; 
        a3 = 0.45449630;
        b3 = 0.04550370;
        coeff(:,1)  = [a1  b1  b1  b1]';
		coeff(:,2)  = [b1  a1  b1  b1]';
		coeff(:,3)  = [b1  b1  a1  b1]';
        coeff(:,4)  = [b1  b1  b1  a1]';   
        coeff(:,5)  = [a2  b2  b2  b2]';
		coeff(:,6)  = [b2  a2  b2  b2]';
		coeff(:,7)  = [b2  b2  a2  b2]';
        coeff(:,8)  = [b2  b2  b2  a2]';   
        coeff(:,9) =  [a3  a3  b3  b3]';        
	    coeff(:,10) = [a3  b3  a3  b3]';    
        coeff(:,11) = [a3  b3  b3  a3]';
        coeff(:,12) = [b3  a3  a3  b3]';
        coeff(:,13) = [b3  a3  b3  a3]';
        coeff(:,14) = [b3  b3  a3  a3]';
       
	    weights(1)  =  0.0122488405; 
        weights(2)  =  0.0122488405; 
        weights(3)  =  0.0122488405;
        weights(4)  =  0.0122488405;
        weights(5)  =  0.018781321;
        weights(6)  =  0.018781321; 
        weights(7)  =  0.018781321; 
        weights(8)  =  0.018781321; 
        weights(9)  =  0.0070910035;  
        weights(10) =  0.0070910035;
        weights(11) =  0.0070910035;
        weights(12) =  0.0070910035;
        weights(13) =  0.0070910035;
        weights(14) =  0.0070910035;        
        weights     =  weights*6; 
    end    
    %   Cools
    if (arg1 == 14) & (arg2 == 51)  % 14 points, different weights and generators, 5th order
        a1 = 0.80291903;
		b1 = 0.06569365;
        a2 = 0.75957268;
        b2 = 0.08014244; 
        a3 = 0.40447533;
        b3 = 0.09552467;
        coeff(:,1)  = [a1  b1  b1  b1]';
		coeff(:,2)  = [b1  a1  b1  b1]';
		coeff(:,3)  = [b1  b1  a1  b1]';
        coeff(:,4)  = [b1  b1  b1  a1]';   
        coeff(:,5)  = [a2  b2  b2  b2]';
		coeff(:,6)  = [b2  a2  b2  b2]';
		coeff(:,7)  = [b2  b2  a2  b2]';
        coeff(:,8)  = [b2  b2  b2  a2]';   
        coeff(:,9) =  [a3  a3  b3  b3]';        
	    coeff(:,10) = [a3  b3  a3  b3]';    
        coeff(:,11) = [a3  b3  b3  a3]';
        coeff(:,12) = [b3  a3  a3  b3]';
        coeff(:,13) = [b3  a3  b3  a3]';
        coeff(:,14) = [b3  b3  a3  a3]';
       
	    weights(1)  =  0.00000973033316198362; 
        weights(2)  =  0.00000973033316198362; 
        weights(3)  =  0.00000973033316198362;
        weights(4)  =  0.00000973033316198362;
        weights(5)  =  0.00899031481668747;
        weights(6)  =  0.00899031481668747;
        weights(7)  =  0.00899031481668747;
        weights(8)  =  0.00899031481668747; 
        weights(9)  =  0.0217777476778781;  
        weights(10) =  0.0217777476778781;
        weights(11) =  0.0217777476778781;
        weights(12) =  0.0217777476778781;
        weights(13) =  0.0217777476778781;
        weights(14) =  0.0217777476778781;                
        weights     =  weights*6; 
    end    
    %   Cools
    if (arg1 == 15) & (arg2==5)   % 5th  order; a1 = 0 makes point on a face of tetrahedron             
	  	a1 = 0.00000000;
		b1 = 0.33333333;
        a2 = 0.72727273;
        b2 = 0.09090909; 
        a3 = 0.06655015;
        b3 = 0.43344984;
        coeff(:,1)  = [1/4  1/4  1/4  1/4]';
		coeff(:,2)  = [a1  b1  b1  b1]';
		coeff(:,3)  = [b1  a1  b1  b1]';
		coeff(:,4)  = [b1  b1  a1  b1]';
        coeff(:,5)  = [b1  b1  b1  a1]';          
        coeff(:,6)  = [a2  b2  b2  b2]';        
	    coeff(:,7)  = [b2  a2  b2  b2]';    
        coeff(:,8)  = [b2  b2  a2  b2]';
        coeff(:,9)  = [b2  b2  b2  a2]';
        coeff(:,10) = [a3  a3  b3  b3]';        
	    coeff(:,11) = [a3  b3  a3  b3]';    
        coeff(:,12) = [a3  b3  b3  a3]';
        coeff(:,13) = [b3  a3  a3  b3]';
        coeff(:,14) = [b3  a3  b3  a3]';
        coeff(:,15) = [b3  b3  a3  a3]';
	    weights(1)  =  0.03028367; 
        weights(2)  =  0.00602678; 
        weights(3)  =  0.00602678;
        weights(4)  =  0.00602678;
        weights(5)  =  0.00602678;
        weights(6)  =  0.01164525; 
        weights(7)  =  0.01164525; 
        weights(8)  =  0.01164525; 
        weights(9)  =  0.01164525;  
        weights(10) =  0.01094914;
        weights(11) =  0.01094914;
        weights(12) =  0.01094914;
        weights(13) =  0.01094914;
        weights(14) =  0.01094914;
        weights(15) =  0.01094914;        
        weights     =  weights*6; 
    end        
    %   Cools
    if (arg1==33) & (arg2==7)                       %  7th order
        a1 = 0.05777714911166000;
        b1 = 0.31407428362944793;
        a2 = 0.81235958507557000;
        b2 = 0.062546804974803940; 
        
        a3 = 0.041202138799372754;
        b3 = 0.61738099120939838;
        c3 = 0.30021473119186;
        
        a4 = 0.25198057383388331;
        b4 = 0.44311853122966217;
        c4 = 0.05292032110257;

        coeff(:,1)  = [1/4  1/4  1/4  1/4]';
        coeff(:,2)  = [a1  b1  b1  b1]';
        coeff(:,3)  = [b1  a1  b1  b1]';
        coeff(:,4)  = [b1  b1  a1  b1]';
        coeff(:,5)  = [b1  b1  b1  a1]';          
        coeff(:,6)  = [a2  b2  b2  b2]';        
        coeff(:,7)  = [b2  a2  b2  b2]';    
        coeff(:,8)  = [b2  b2  a2  b2]';
        coeff(:,9)  = [b2  b2  b2  a2]'; 
        coeff(:,10) = [a3  a3  b3  c3]';    
        coeff(:,11) = [a3  a3  c3  b3]';
        coeff(:,12) = [a3  b3  a3  c3]';
        coeff(:,13) = [a3  c3  a3  b3]';
        coeff(:,14) = [a3  b3  c3  a3]';
        coeff(:,15) = [a3  c3  b3  a3]';
        coeff(:,16) = [b3  a3  a3  c3]';
        coeff(:,17) = [b3  c3  a3  a3]';
        coeff(:,18) = [b3  a3  c3  a3]';   
        coeff(:,19) = [c3  a3  a3  b3]';
        coeff(:,20) = [c3  b3  a3  a3]';
        coeff(:,21) = [c3  a3  b3  a3]';
        coeff(:,22) = [a4  a4  b4  c4]';    
        coeff(:,23) = [a4  a4  c4  b4]';
        coeff(:,24) = [a4  b4  a4  c4]';
        coeff(:,25) = [a4  c4  a4  b4]';
        coeff(:,26) = [a4  b4  c4  a4]';
        coeff(:,27) = [a4  c4  b4  a4]';
        coeff(:,28) = [b4  a4  a4  c4]';
        coeff(:,29) = [b4  c4  a4  a4]';
        coeff(:,30) = [b4  a4  c4  a4]';   
        coeff(:,31) = [c4  a4  a4  b4]';
        coeff(:,32) = [c4  b4  a4  a4]';
        coeff(:,33) = [c4  a4  b4  a4]';

        weights(1)  =  0.017783308938202055; 
        weights(2)  =  -0.019099406118263309;
        weights(3)  =  -0.019099406118263309;
        weights(4)  =  -0.019099406118263309;
        weights(5)  =  -0.019099406118263309;
        weights(6)  =  3.8252276536630236*1e-3; 
        weights(7)  =  3.8252276536630236*1e-3;  
        weights(8)  =  3.8252276536630236*1e-3;  
        weights(9)  =  3.8252276536630236*1e-3;  
        weights(10) =  3.5475499144103265*1e-3; 
        weights(11) =  3.5475499144103265*1e-3;
        weights(12) =  3.5475499144103265*1e-3;
        weights(13) =  3.5475499144103265*1e-3;
        weights(14) =  3.5475499144103265*1e-3;
        weights(15) =  3.5475499144103265*1e-3;
        weights(16) =  3.5475499144103265*1e-3; 
        weights(17) =  3.5475499144103265*1e-3;
        weights(18) =  3.5475499144103265*1e-3;
        weights(19) =  3.5475499144103265*1e-3;
        weights(20) =  3.5475499144103265*1e-3;
        weights(21) =  3.5475499144103265*1e-3;
        weights(22) =  0.013950789384495152;
        weights(23) =  0.013950789384495152;
        weights(24) =  0.013950789384495152;
        weights(25) =  0.013950789384495152;
        weights(26) =  0.013950789384495152;
        weights(27) =  0.013950789384495152;
        weights(28) =  0.013950789384495152;
        weights(29) =  0.013950789384495152;
        weights(30) =  0.013950789384495152;
        weights(31) =  0.013950789384495152;
        weights(32) =  0.013950789384495152;
        weights(33) =  0.013950789384495152;        
        weights     =  weights*6; 
    end    
    %   Cools
    if (arg1==53) & (arg2==9)                       %  9th order        
        a1 = 0.854946884350790000;
        b1 = 0.048351038549736740;
        
        a2 = 0.02626215964635000;
        b2 = 0.32457928011788236; 
        
        a3 = 0.65615037932801000;
        b3 = 0.11461654022399521; 
        
        a4 = 0.32353014426546000;
        b4 = 0.22548995191151391;         
        
        a5 = 0.131627809246869800;
        b5 = 0.083664701617184967;
        c5 = 0.653079679889080000;
        
        a6 = 0.43395146141140677;
        b6 = 0.10776985954942861;
        c6 = 0.02432721762776000;        

        a7 = -1.3762773181382007e-3;
        b7 = 0.27655347263680734;
        c7 = 0.72619908199947;     
        
        coeff(:,1)  = [1/4  1/4  1/4  1/4]';
        
        coeff(:,2)  = [a1  b1  b1  b1]';
        coeff(:,3)  = [b1  a1  b1  b1]';
        coeff(:,4)  = [b1  b1  a1  b1]';
        coeff(:,5)  = [b1  b1  b1  a1]';      
        
        coeff(:,6)  = [a2  b2  b2  b2]';        
        coeff(:,7)  = [b2  a2  b2  b2]';    
        coeff(:,8)  = [b2  b2  a2  b2]';
        coeff(:,9)  = [b2  b2  b2  a2]'; 

        coeff(:,10)  = [a3  b3  b3  b3]';        
        coeff(:,11)  = [b3  a3  b3  b3]';    
        coeff(:,12)  = [b3  b3  a3  b3]';
        coeff(:,13)  = [b3  b3  b3  a3]';         
        
        coeff(:,14)  = [a4  b4  b4  b4]';        
        coeff(:,15)  = [b4  a4  b4  b4]';    
        coeff(:,16)  = [b4  b4  a4  b4]';
        coeff(:,17)  = [b4  b4  b4  a4]';         
        
        coeff(:,18) = [a5  a5  b5  c5]';    
        coeff(:,19) = [a5  a5  c5  b5]';
        coeff(:,20) = [a5  b5  a5  c5]';
        coeff(:,21) = [a5  c5  a5  b5]';
        coeff(:,22) = [a5  b5  c5  a5]';
        coeff(:,23) = [a5  c5  b5  a5]';
        coeff(:,24) = [b5  a5  a5  c5]';
        coeff(:,25) = [b5  c5  a5  a5]';
        coeff(:,26) = [b5  a5  c5  a5]';   
        coeff(:,27) = [c5  a5  a5  b5]';
        coeff(:,28) = [c5  b5  a5  a5]';
        coeff(:,29) = [c5  a5  b5  a5]';
        
        coeff(:,30) = [a6  a6  b6  c6]';    
        coeff(:,31) = [a6  a6  c6  b6]';
        coeff(:,32) = [a6  b6  a6  c6]';
        coeff(:,33) = [a6  c6  a6  b6]';
        coeff(:,34) = [a6  b6  c6  a6]';
        coeff(:,35) = [a6  c6  b6  a6]';
        coeff(:,36) = [b6  a6  a6  c6]';
        coeff(:,37) = [b6  c6  a6  a6]';
        coeff(:,38) = [b6  a6  c6  a6]';   
        coeff(:,39) = [c6  a6  a6  b6]';
        coeff(:,40) = [c6  b6  a6  a6]';
        coeff(:,41) = [c6  a6  b6  a6]';        
       
        coeff(:,42) = [a7  a7  b7  c7]';    
        coeff(:,43) = [a7  a7  c7  b7]';
        coeff(:,44) = [a7  b7  a7  c7]';
        coeff(:,45) = [a7  c7  a7  b7]';
        coeff(:,46) = [a7  b7  c7  a7]';
        coeff(:,47) = [a7  c7  b7  a7]';
        coeff(:,48) = [b7  a7  a7  c7]';
        coeff(:,49) = [b7  c7  a7  a7]';
        coeff(:,50) = [b7  a7  c7  a7]';   
        coeff(:,51) = [c7  a7  a7  b7]';
        coeff(:,52) = [c7  b7  a7  a7]';
        coeff(:,53) = [c7  a7  b7  a7]';        

        
        weights(1)  =  -0.13779903832610864;
        
        weights(2)  =  1.8653365690852895e-3;
        weights(3)  =  1.8653365690852895e-3;
        weights(4)  =  1.8653365690852895e-3;
        weights(5)  =  1.8653365690852895e-3;     
        
        weights(6)  =  4.3094239694934006e-3; 
        weights(7)  =  4.3094239694934006e-3;  
        weights(8)  =  4.3094239694934006e-3;  
        weights(9)  =  4.3094239694934006e-3;          
        
        weights(10)  =  -0.090184766481201525; 
        weights(11)  =  -0.090184766481201525;  
        weights(12)  =  -0.090184766481201525;  
        weights(13)  =  -0.090184766481201525;  
        
        weights(14)  =  0.044672576202511444; 
        weights(15)  =  0.044672576202511444;  
        weights(16)  =  0.044672576202511444;  
        weights(17)  =  0.044672576202511444;                  
        
        weights(18:29) =  0.034700405884550761;         
        weights(30:41) =  3.3525839026606469e-3; 
        weights(42:53) =  4.3162887555699692e-4;   
        
        weights     =  weights*6; 
    end    
    
end
IndexT = size(coeff,2);