function [charge chargev] = charged(GEOM, D);
%CHARGED - Outputs the surface charge density for the dielectric faces. 
%
% Syntax:
%   [charge chargev] = charged(GEOM, D);
%
% Description:
%   CHARGED computes the bound surface charge density on the dielectric faces.
%
% Inputs:
%   GEOM - structure array that defines all the parameters of the
%       dielectric tetrahedra including basis functions
%   D - MoM solution vector
%
% Outputs:
%   charge  - array of charges for dielectric faces
%   chargev - interpolated array for vertexes
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov

charge = zeros(1, GEOM.FacesNontrivial);
for n = 1:GEOM.FacesNontrivial 
     for c = 1:GEOM.EdgesFNI(n)  
        N = GEOM.EdgesFN(c, n);  %   base edge (global number)
        charge(n) = charge(n) + D(N)*(GEOM.DiffContrast_real(c,n)+ j*GEOM.DiffContrast_imag(c,n))/GEOM.AreaF(n); 
     end
end

%   Interpolation for vertexes
chargev = zeros(1, length(GEOM.P));
for m = 1:size(GEOM.P, 2)
    chargev(m)      = 0;
    [p, q]          = find(GEOM.Faces(:,1:GEOM.FacesNontrivial)-m==0);
    if (length(q))
        chargev(m)      = sum(charge(:,q), 2)/length(q);
    end
end   
