function [charge, chargev] = chargem(geom, I, freq);
%CHARGEM - Computes free charge density on the metal faces
%
% Syntax:
%   [charge chargev] = chargem(geom, I, freq);
%
% Description:
%   CHARGEM calculates the free charge density on the metal faces according to RWG
%   definition
%
% Inputs:
%   geom - structure array that defines all the parameters of the
%       metal faces including RWG basis functions
%   I - MoM solution for metal currents
%   freq - real part of frequency
%
% Outputs:
%   charge  - array of charges for metal faces
%   chargev - interpolated array for vertexes
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov

omega = 2*pi*freq;

charge  = zeros(1, geom.TrianglesTotal);

for m = 1:geom.EdgesTotal
    IE = I(m)*geom.EdgeLength(m);
    charge(geom.TriP(m)) = charge(geom.TriP(m)) + IE/(geom.AreaF(geom.TriP(m)));
    charge(geom.TriM(m)) = charge(geom.TriM(m)) - IE/(geom.AreaF(geom.TriM(m)));
end
  
charge  = charge/(-j*omega);

%   Interpolation for vertexes
chargev = zeros(1, length(geom.P));
for m = 1:size(geom.P, 2)
    chargev(m)      = 0;
    [p, q]          = find(geom.t(1:3,:)-m==0);
    if (length(q))
        chargev(m)      = sum(charge(:,q), 2)/length(q);
    end
end   

%   Test:
%   CE      = mean(charge.*geom.AreaF)/mean(abs(charge.*geom.AreaF))
%   will be exactly zero