function [current, currentv] =currentm(geom, I);
%CURRENTM - Computes surface current density on the metal faces 
%
% Syntax:
%   [current, currentv] =currentm(geom, I);
%
% Description:
%   CURRENTM calculates the surface current density on the metal faces according to RWG
%   definition
%
% Inputs:
%   geom - structure array that defines all the parameters of the
%       metal faces including RWG basis functions
%   I - MoM solution for metal currents
%
% Outputs:
%   current  - array of charges for metal faces
%   currentv - interpolated array for vertexes
%
% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov


current = zeros(3,geom.TrianglesTotal);
for m= 1:geom.EdgesTotal
    TP      = geom.TriP(m);
    TM      = geom.TriM(m);
    rhoP    = + geom.PointsFCRho(:,geom.VerP(m),TP);
    rhoM    = - geom.PointsFCRho(:,geom.VerM(m),TM);
    current (:,TP)  = current(:,TP) + I(m)*rhoP*geom.EdgeLength(m)/(2*geom.AreaF(TP));
    current(:,TM)   = current(:,TM) + I(m)*rhoM*geom.EdgeLength(m)/(2*geom.AreaF(TM));
end       

%   Interpolation for vertexes
currentv = zeros(3, length(geom.P));
for m = 1:size(geom.P,2)
    [p,q]           = find(geom.t(1:3,:)-m==0);
    if (length(q))
        currentv(:,m)   = sum(current(:,q), 2)/length(q);
    end    
end   
