function [E, H, Poynting]    = field(geom, GEOM, const, output, f_re, Points, flag);
% FIELD - Computes electric field, magnetic field, and Poynting vector of
% the metal-dielectric structure
%
% Syntax:
%   [E, H, Poynting] = field(geom,, GEOM, const, output, f_re, Points, flag);
%                      
% Description:
%   FIELD Computes scattered electric field, magnetic field, and Poynting vector of
%   the metal-dielectric structure. 
%
% Inputs:
%   geom    - structure array that defines the RWG basis functions
%   GEOM    - structure array that defines all the parameters of the
%       dielectric tetrahedra including basis functions
%   const   - structure array defining electromagnetic parameters
%   output  - MoM solution vector
%   f_re    - frequency of operation in Hz (real frequency only)
%   Points  - 3-by-N array of observation points where the fields are 
%   to be computed
%   flag    - dummy parameter, should be set to zero.   
%   More detailed information on using FIELD can be found in the
%   documentation
%
% Outputs:
%
%   E - array of scattered electric field (complex vector at a point, V/m)
%   H - array of scattered magnetic field (complex vector at a point, A/m)            
%   Poynting - array of the Poynting vector (W/m^2) for scattered field
%   All are of size (3-by-Points where the field is calculated).

% MATLAB Antenna Toolbox Draft 05, Antenna Lab, ECE, WPI, mat05@ece.wpi.edu  
% Authors: S. Kulkarni, A. Marut, S. Makarov


EM        = repmat([0 0 0]',[1, size(Points,2)]);
ED        = repmat([0 0 0]',[1, size(Points,2)]);
HM        = repmat([0 0 0]',[1, size(Points,2)]);
HD        = repmat([0 0 0]',[1, size(Points,2)]);

Radius = 2;     % tested - all right (this value is not important for far fields)
                % (dimensionless radius of the sphere inside which the fields are integrated precisely)
dummy  = 0;

if ~isempty(geom)
    dummy   = geom.EdgesTotal;
    I       =  output(1:dummy);
    [PoyntingM, EM, HM]  = fieldm(const, geom, I, f_re, Points, Radius, flag);
    
     
end
if ~isempty(GEOM)
    D =  output(dummy+1:end);
    [PoyntingD, ED, HD]  =  fieldd(const, GEOM, D, f_re,  Points, Radius, flag, 1); 
end
             
E = EM + ED;
H = HM + HD;

Poynting = 0.5*real(cross(E,conj(H)));
