%RCS_PATTERNS - bistatic radar cross-section patterns
%
% Syntax:
%   rcs_patterns
%
% Description:
%   RCS_PATTERNS find the 3D bistatic RCS patterns as defined in C. A.
%   Balanis, Advanced Eng. Electromagnetics, Wiley, 1989, p. 578, Fig. 11-16.  
%   sigma_3D = lim(4*pi*r^2*abs(Es)^2/abs(Ei)^2));
%
%   Note: to employ MATLAB polar plot for RCS patterns please use 
%   arg = pi/2-theta for x>0  and arg=pi/2+theta for x<0   
clear all;

s = pwd;  cd ..;
pathadd(s, 0); cd(s);
cd ..; load  ('2_basis/basis');
cd  ('3_mom/'); 

%   Define frequency
freq            = 300e6;
lambda          = const.c/freq; 
omega_re        = 2*pi*freq;
omega_im        = 0;

%   Define the incident plane wave (TMz polarization)
thetai   = pi/4;    %   elevation angle of incidence  
E0       = [cos(thetai) 0  sin(thetai)]';   %   TMz
% E0       = [0 1 0]';                      %   TEz    
k        = omega_re/const.c;
dir0     = [sin(thetai) 0 -cos(thetai)]'*omega_re/const.c;

%   Define the excitation voltage vector
VM = [];  VD = [];
if ~isempty(geom)
    pol     = repmat(E0, [1, geom.TrianglesTotal]);
    dir     = repmat(dir0, [1, geom.TrianglesTotal]);
    kr      = dot(dir, geom.PointsFC, 1);
    Einc    = pol.*repmat(exp(-j*kr), [3, 1]) + j*1e-64;
    VM      = voltagem(geom, Einc);    
end
if ~isempty(GEOM)
    pol     = repmat(E0, [1, GEOM.TetrahedraTotal]);
    dir     = repmat(dir0, [1, GEOM.TetrahedraTotal]);
    kr      = dot(dir, GEOM.CenterT, 1);
    Einc    = pol.*repmat(exp(-j*kr), [3, 1]) + j*1e-64;
    VD      = voltaged(GEOM, Einc);
end
V = [VM; VD];

%   MoM solve
output      = zmdslv(geom, GEOM, const, 2*pi*freq, 0.0, 1, V, []);  
output      = j*(2*pi*freq)*output;

%   Define the elevation planes
R       = 100;                                      %   radius of observation sphere in meters
P       = 180;                                      %   angular grid resolution
thetas  = [[0:pi/P:pi-pi/P] [pi:-pi/P:0]];          %   full thetas (0->pi->0)
phi0    = 0.0;                                      %   this is the x-plane
phi     = [ones(1, P)*phi0 ones(1, P+1)*(phi0+pi)]; %   full phi (phi0 and phi0+pi)

%   Pattern for both forward scattering (x>0) and backscattering (x<0)
z       = R*cos(thetas);
x       = R*sin(thetas).*cos(phi);
y       = R*sin(thetas).*sin(phi);
Points  = [x; y; z];
flag    = 0;   % only scattered field
[E, H, Poynting]= field(geom, GEOM, const, output, freq, Points, flag);

%   Etheta and Ephi polarizations
factor1 = cos(thetas).*cos(phi);
factor2 = cos(thetas).*cos(phi);
factor3 = sin(thetas);
Etheta  =  E(1,:).*factor1  + E(2,:).*factor2 - E(3,:).*factor3;
Ephi    = -E(1,:).*sin(phi) + E(2,:).*cos(phi);
Htheta  =  H(1,:).*factor1  + H(2,:).*factor2 - H(3,:).*factor3;
Hphi    = -H(1,:).*sin(phi) + H(2,:).*cos(phi);

%   Right-handed (ER) and left-handed (EL) CP polarizations
ER      = 1/sqrt(2)*(Etheta + j*Ephi);
EL      = 1/sqrt(2)*(Etheta - j*Ephi);

%   RCS pattern 
Component  =  Hphi*const.eta;                           %   TMz
%   Component  =  Ephi;                                 %   TEz
RCS        =  4*pi*R^2*dot(Component, Component, 1);    %   bistatic RCS
RCS        = 10*log10(RCS./lambda^2);                   %   RCS in dB
RCS(find(RCS <= 0)) = 0;                                %   find and eliminate negative values 
thetam = [pi/2-thetas(1:P) pi/2+thetas(P+1:2*P+1)];     %   MATLAB argument for the polar plot
polar(thetam, RCS, 'r');
grid on

s = pwd;
cd ..;
pathremove;
cd(s);
