%RCS_PATTERNS - monostatic radar cross-section patterns
%
% Syntax:
%   rcs_patterns_mono
%
% Description:
%   RCS_PATTERNS_MONO finds the 3D monotatic RCS patterns as defined in C. A.
%   Balanis, Advanced Eng. Electromagnetics, Wiley, 1989, p. 578, Fig. 11-16.  
%   sigma_3D = lim(4*pi*r^2*abs(Es)^2/abs(Ei)^2));
%
%   Note: to employ MATLAB polar plot for RCS patterns please use 
%   arg = pi/2-theta for x>0  and arg=pi/2+theta for x<0   
clear all;

s = pwd;  cd ..;
pathadd(s, 0); cd(s);
cd ..; load  ('2_basis/basis');
cd  ('3_mom/'); 

%   Define frequency
freq            = 300e6;    %   wavelength is 1m
lambda          = const.c/freq; 
omega_re        = 2*pi*freq;
omega_im        = 0;

%   Define the incident plane waves (TMz polarization)
%   Define the excitation voltage vectors
thetai   = [0:1:50]/180*pi;      %   elevation angles of incidence  
V        = [];
for m = 1: length(thetai)
    E0       = [cos(thetai(m)) 0  sin(thetai(m))]';   %   TMz
    E0       = [0              1        0       ]';   %   TEz
    k        = omega_re/const.c;
    dir0     = [sin(thetai(m)) 0 -cos(thetai(m))]'*omega_re/const.c;    
    VM       = [];  
    VD       = [];
    if ~isempty(geom)
        pol     = repmat(E0, [1, geom.TrianglesTotal]);
        dir     = repmat(dir0, [1, geom.TrianglesTotal]);
        kr      = dot(dir, geom.PointsFC, 1);
        Einc    = pol.*repmat(exp(-j*kr), [3, 1]) + j*1e-64;
        VM      = voltagem(geom, Einc);    
    end
    if ~isempty(GEOM)
        pol     = repmat(E0, [1, GEOM.TetrahedraTotal]);
        dir     = repmat(dir0, [1, GEOM.TetrahedraTotal]);
        kr      = dot(dir, GEOM.CenterT, 1);
        Einc    = pol.*repmat(exp(-j*kr), [3, 1]) + j*1e-64;
        VD      = voltaged(GEOM, Einc);
    end
    V = [V; [VM; VD]];
end

%   MoM solve
output      = zmdslv(geom, GEOM, const, 2*pi*freq, 0.0, length(thetai), V, []);  
output      = j*(2*pi*freq)*output;
Current     = reshape(output, [geom.EdgesTotal, length(thetai)]);

%   Monostatic RCS
for m = 1:length(thetai)
    %   Define the observation point
    thetas  = thetai(m);          
    phi     = pi;                           %   this is backscattering (x<0)
    R       = 10000;                        %   in meters
    z       = R*cos(thetas);
    x       = R*sin(thetas).*cos(phi);
    y       = R*sin(thetas).*sin(phi);
    Points  = [x; y; z];
    flag    = 0;   % only scattered field
    [E, H, Poynting]= field(geom, GEOM, const, Current(:, m), freq, Points, flag);
    %   Etheta and Ephi polarizations
    factor1 = cos(thetas).*cos(phi);
    factor2 = cos(thetas).*cos(phi);
    factor3 = sin(thetas);
    Etheta  =  E(1,:).*factor1  + E(2,:).*factor2 - E(3,:).*factor3;
    Ephi    = -E(1,:).*sin(phi) + E(2,:).*cos(phi);
    Htheta  =  H(1,:).*factor1  + H(2,:).*factor2 - H(3,:).*factor3;
    Hphi    = -H(1,:).*sin(phi) + H(2,:).*cos(phi);
    %   Right-handed (ER) and left-handed (EL) CP polarizations
    ER      = 1/sqrt(2)*(Etheta + j*Ephi);
    EL      = 1/sqrt(2)*(Etheta - j*Ephi);
    %   RCS pattern 
    Component  =  Etheta;	%	TMz  
    Component  =  Ephi;		%	TEz                          
    RCS        =  4*pi*R^2*dot(Component, Component, 1);    %   bistatic RCS
    RCSmono(m) = 10*log10(RCS./lambda^2);                   %   RCS in dB
end
plot(thetai*180/pi, RCSmono, '-bs', 'LineWidth', 2,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',10);
grid on;
title ('Monostatic RCS for Etheta, dB'); xlabel('\theta, deg');
ylabel('RCS, dB'); axis([min(thetai*180/pi) max(thetai*180/pi) 0 50]);

thisfile = strcat('TE_Triangles=', num2str(geom.TrianglesTotal));
save(thisfile, 'thetai', 'RCSmono');

s = pwd;
cd ..;
pathremove;
cd(s);
